/* File Name(s)  : ecia1.c

   Product       : CICS Transaction Gateway

   Description   : This sample shows an advanced form of using the CICS TG C
                   API. It queries the Client daemon for a list of available  
                   servers and then creates a number of threads per server.
                   Each thread runs a number of ECI calls on that server
                   asynchronously and works out the average response time per
                   server

   Prerequisites:  This sample requires the CICS Program EC01 to be
                   installed on your CICS server for operation.

 Licensed Materials - Property of IBM  
  
 5724-I81,5725-B65 
  
 (C) Copyright IBM Corp. 1999, 2014 All Rights Reserved.  
  
 US Government Users Restricted Rights - Use, duplication or  
 disclosure restricted by GSA ADP Schedule Contract with  
 IBM Corp.  
   
   Status: Version 9 Release 1 

                   The following code is sample code created by IBM
                   Corporation.  This sample code is not part of any
                   standard IBM product and is provided to you solely for
                   the purpose of assisting you in the development of your
                   applications.  The code is provided 'AS IS', without
                   warranty or condition of any kind.  IBM shall not be
                   liable for any damages arising out of your use of the
                   sample code, even if IBM has been advised of the
                   possibility of such damages. */


/* Include files */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <cics_eci.h>
#include <cclcalls.h>

/* Structures, defines and macro definitions */

struct RunInfo
{
        int TotalThreads;
        int NumCompleteOK;
        int NumCompleteError;
};

#define ECI_SYNC_SIZE 18
#define MAX_SYSTEMS   20

#define CCLLOG(A)        printf("%d (%d,%d)=> %s\n", QueryMyThreadId(), Idx,     \
                                 Thread, A);
#define CCLLOG2(A,B,C,D) printf("%d (%d,%d)=> ", QueryMyThreadId(), Idx, Thread);\
                         printf(A,B,C,D);                                        \
                         printf("\n");

#define SYNC_OPTION		"SYNC"
#define ASYNC_OPTION	"ASYNC"

#if defined(CICS_W32)
#define DEBUG_OPTION	"/DEBUG"
#else
#define DEBUG_OPTION	"-DEBUG"
#endif

#define copyright_notice copyright4cecia1
char version4cecia1[] =
"\nCICS Transaction Gateway"
" Version 9.1.0.4"
" Build Level c910-20231012-1355" ;

char copyright_notice[] =
"\n"
"Licensed Materials - Property of IBM. "
"\n"
"5724-I81,5725-B65 "
"\n"
"(c) Copyright IBM Corp. 1999, 2014 All Rights Reserved. "
"\n"
"US Government Users Restricted Rights - Use, duplication or "
"disclosure restricted by GSA ADP Schedule Contract with "
"IBM Corp."
"\n";

/*  Global Variables - If accessed by multiple threads, must be protected by
                       a mutex semaphore */

cics_ushort_t    serverCount = MAX_SYSTEMS; /*Actual no of servers in ini file*/
CICS_EciSystem_t servers[MAX_SYSTEMS];      /*CICS_EciListSystems Server list */
float           *average;                   /*Average time per call so far    */
CclEvent         StartTest,                 /*Event flagged to start the test */
                 FinishTest,                /*Event flagged by last srv thread*/
                 StartedThread;             /*Event flagged as a thread starts*/
CclEvent        *Callback;                  /*Array of events for async calls */

/* Saved copies of initial args */

int              ThreadsPerServer= 5;       /* Threads per server (Default: 1)*/
int              CallsPerThread  = 10;      /* No calls per thread(Default:10)*/
CclBool          Debug           = CCLFALSE;/* Display Debug information      */
CclBool          AsyncCalls      = CCLFALSE;/* Are we sync/async              */

struct RunInfo Progress; /* Tracks threads started/completed */
CclMutex ProgressLock; /* Protects update of Progress object */


/* Function prototypes */

void Response(short Rc, cics_char_t *Abend, CclThreadId Tid);
char *StrToupper(char *str);

#if defined CICS_W32
void __cdecl ECILoop(void *IdxPtr);
#else
void ECILoop(void *IdxPtr);
#endif
void CICSEXIT CallBackProc (cics_ushort_t AIdx);
void AbortThread(void);


/* Main Function */

int main(int argc, char *argv[]){
   short rc;
   struct Parms {
      int i;
      int j;
   } Parm;
   float RunningTotal;
   int   Size;
   char  TermInput[100];


   /* Display Banner and syntax of program */

   printf("CICS Transaction Gateway Advanced ECI V1 Sample\n\n");
   printf("SYNTAX : \n");
   printf("  ecia1 <Threads> <Calls> <%s|%s> [%s]\n", SYNC_OPTION, ASYNC_OPTION, DEBUG_OPTION);
   printf("    where \n");
   printf("  Threads    = Number of threads to run per server (Default=5)\n");
   printf("  Calls      = Number of ECI calls to make per thread (Default=10)\n");
   printf("  SYNC/ASYNC = Do (a)synchronous eci calls (Default=SYNC)\n");
   printf("  %s     = Show all messages (Default is off)\n", DEBUG_OPTION);
   printf("\n");


   /* Parsing of the args */

   if (argc > 1) {
	   if(atol(argv[1]) < 1) {
		   printf("ERROR: \"%s\" %s", argv[1], " is not a valid choice, threads per server needs to be above 0\n");
		   return -1;
	   }
       else ThreadsPerServer = atol(argv[1]);
   }
   if (argc > 2) {
	  if(strcmp(argv[2], "0") == 0 || atol(argv[2]) > 0) CallsPerThread   = atol(argv[2]);
	  else {
		  printf("ERROR: \"%s\" %s", argv[2], " is not a valid choice, ECI calls per thread needs to be 0 or above\n");
		  return -1;
	  }
   }
   if (argc > 3) {
      if (strcmp(StrToupper(argv[3]), ASYNC_OPTION) == 0) AsyncCalls = CCLTRUE;
	  else if(strcmp(StrToupper(argv[3]), SYNC_OPTION) == 0) AsyncCalls = CCLFALSE;
	  else {
		  printf("ERROR: \"%s\" is not a valid choice\n", argv[3]);
		  return -1;
	  }
   }
   if (argc > 4) {
	   if(strcmp(StrToupper(argv[4]), DEBUG_OPTION) == 0) Debug = CCLTRUE;
	   else {
		  printf("ERROR: \"%s\" is not a valid choice\n", argv[4]);
		  return -1;
	  }
   }


   /* Ask the Client daemon for a list of servers to connect to */

   rc = CICS_EciListSystems(NULL,&serverCount,servers);
   if (rc != ECI_NO_ERROR) {
      Response(rc, NULL, 0);
      if (rc != ECI_ERR_MORE_SYSTEMS) return 1;
   }


   /* Display the parameters which will be used */

   printf("Running with: \n");
   printf("%d servers\n", serverCount);
   printf("%d threads per server\n", ThreadsPerServer);
   printf("%d ECI calls per thread\n", CallsPerThread);

   if (AsyncCalls == CCLTRUE) {
      printf("Call type is asynchronous\n");
   } else {
      printf("Call type is synchronous\n");
   }


   /* Create a Mutex and some Event semaphores */

   if (CreateAnEvent(NULL, &StartedThread) != CCLTRUE) {
      printf("MAIN : Failed to create thread started event - aborting process\n");
      return 1;
   }
   if (CreateAMutex(NULL, &ProgressLock) != CCLTRUE) {
      printf("MAIN : Failed to create main mutex - aborting process\n");
      return 1;
   }
   if (CreateAnEvent(NULL, &StartTest) != CCLTRUE) {
      printf("MAIN : Failed to create StartTest event - aborting process\n");
      return 1;
   }
   if (CreateAnEvent(NULL, &FinishTest) != CCLTRUE) {
      printf("MAIN : Failed to create FinishTest event - aborting process\n");
      return 1;
   }


   /* Malloc some storage for the averages and events used if async */

   Size = sizeof(float) * serverCount * ThreadsPerServer;
   average = malloc(Size);
   if (average == CCLNULL) {
      printf("MAIN : Failed to malloc storage - aborting process\n");
      return 1;
   }

   if (AsyncCalls == CCLTRUE) {
      Size = sizeof(CclEvent) * serverCount * ThreadsPerServer;
      Callback = malloc(Size);
      if (Callback == CCLNULL) {
         printf("MAIN : Failed to malloc storage for events - aborting process\n");
         return 1;
      }
   }

   Progress.NumCompleteError = 0;
   Progress.NumCompleteOK = 0;
   Progress.TotalThreads = serverCount * ThreadsPerServer;


   /* Start the required number of threads for each server */

   for (Parm.i=0; Parm.i<serverCount; Parm.i++) {
      for (Parm.j=0; Parm.j<ThreadsPerServer; Parm.j++) {

         /* Start a new thread */

         if (StartAThread( ECILoop, &Parm ) != CCLTRUE) {
            printf("MAIN : Failed to create thread %d/%d - aborting process\n",
                    Parm.i, Parm.j);
            return 1;
         }

         /* Wait for it to completely start */

         if (Debug == CCLTRUE) printf("MAIN : Waiting for new thread to start\n");
         if (WaitOnAnEvent( &StartedThread ) != CCLTRUE) {
            printf("MAIN : Failed to wait for thread %d/%d to start - aborting process\n",
                    Parm.i, Parm.j);
            return 1;
         }

         /* Reset the event so it can be reused */

         if (ResetAnEvent( &StartedThread ) != CCLTRUE) {
            printf("MAIN : Failed to wait for thread %d/%d to start - aborting process\n",
                    Parm.i, Parm.j);
            return 1;
         }
      }
   }

   printf("MAIN : Threads all started - starting test\n");


   /* All threads have started and are waiting to go. Post the StartTest event*/

   if (FlagAnEvent( &StartTest ) != CCLTRUE) {
      printf("MAIN : Failed to post the StartTest flag - aborting process\n");
      return 1;
   }


   /* Give the test a second to complete and then wait on the finish event */

   WaitInMillisecs(1000);

   if (WaitOnAnEvent( &FinishTest ) != CCLTRUE) {
      printf("MAIN : Failed to wait on the FinishTest flag - aborting process\n");
      return 1;
   }


   /* Display the tests results */

   printf("MAIN : Threads all finished - results follow:\n");
   if (Progress.NumCompleteError > 0)
   {
      printf("MAIN : %d of %d threads completed in error, results may be affected\n", Progress.NumCompleteError, Progress.TotalThreads);
   }

   for (Parm.i=0; Parm.i<serverCount; Parm.i++) {
      RunningTotal = 0.0;
      for (Parm.j=0; Parm.j<ThreadsPerServer; Parm.j++) {
         if (Debug == CCLTRUE) {
            printf("Server : %s, thread %d average time in seconds %f\n",
                    servers[Parm.i].SystemName,
                    Parm.j,
                    ((float) average[(Parm.i*ThreadsPerServer)+Parm.j])/1000.0);
         }
         RunningTotal = RunningTotal + average[(Parm.i*ThreadsPerServer)+Parm.j];
      }
      printf("Server : %s, overall average time in seconds %f\n",
             servers[Parm.i].SystemName,
             RunningTotal / (1000.0 * (float) ThreadsPerServer));
   }

   printf("Press enter to terminate\n");
   fgets(TermInput, 100, stdin);


   /* Clean up malloced storage and created events/mutexes */

   free(average);
   if (AsyncCalls == CCLTRUE) free(Callback);

   DeleteAnEvent( &StartTest );
   DeleteAnEvent( &FinishTest );
   DeleteAnEvent( &StartedThread );
   DeleteAMutex(  &ProgressLock );

   return 0;
}


/* Function started once running on each new thread */

void ECILoop(void *IdxPtr) {
   char          CommArea[ECI_SYNC_SIZE]; /*COMMAREA for use in ECI calls       */
   char          UserID[9] = "sysad   ";  /*User ID padded to eight characters  */
   char          PassWd[9] = "sysad   ";  /*Password padded to eight characters */
   cics_char_t   progid[9] = "EC01";      /*Program to call padded to 8 chars   */
   ECI_PARMS     EciParms;                /*ECI Parameter block                 */
   short         rc;                      /*ECI Return code                     */
   CclBool       AllFinished = CCLFALSE;  /*Set true if this is the last thread */

   char          CallBackEvt[20];
   struct Parms {
      int        IdxOfServer;
      int        ThreadPerServer;
   } *Parm;

   int           Idx, Thread, AIdx;
   int           i   = 0;

   unsigned long TimeBefore, TimeAfter;


   /* Extract information from main thread which will change */

   Parm    = (struct Parms *) IdxPtr;
   Idx     = Parm->IdxOfServer;
   Thread  = Parm->ThreadPerServer;
   AIdx    = (Idx * ThreadsPerServer) + Thread;

   if (Debug == CCLTRUE) CCLLOG("Thread started");


   /* Inform main thread we have started and obtained all information we need */

   if (FlagAnEvent( &StartedThread) != CCLTRUE) {
      CCLLOG("Failed to inform main thread via StartedThread - aborting process");
      exit(2); /* Fatal error */
   }


   /* Wait for all other threads and for the test to start */

   if (WaitOnAnEvent( &StartTest ) != CCLTRUE) {
      CCLLOG("Failed to wait on the StartTest flag - aborting thread");
      AbortThread();
      EndMyThread();
      return;
   }


   /* Initialize the COMMAREA, average and create an event for the callback */

   average[AIdx] = 0.0;
   memset(CommArea, '\0', ECI_SYNC_SIZE);

   if (AsyncCalls == CCLTRUE) {
      sprintf(CallBackEvt, "Callback%d", AIdx);
      if (CreateAnEvent(CallBackEvt, &Callback[AIdx]) != CCLTRUE) {
         CCLLOG("Failed to create callback event - aborting thread");
         AbortThread();
         EndMyThread();
         return;
      }
   }


   /* Loop running the call the specified number of times */

   if (Debug == CCLTRUE) CCLLOG("Running the test");

   for (i = 0; i<CallsPerThread; i++) {


      /* Set up the ECI parameter block */

      memset(&EciParms, 0, sizeof(ECI_PARMS));

      EciParms.eci_version                       = ECI_VERSION_1A;
      if (AsyncCalls == CCLTRUE) {
         EciParms.eci_call_type                  = ECI_ASYNC;
         EciParms.eci_callback                   = (CICS_EciNotify_t)CallBackProc;
         EciParms.eci_message_qualifier          = (short) AIdx;
      } else {
         EciParms.eci_call_type                  = ECI_SYNC;
      }
      memcpy(&EciParms.eci_program_name, progid,       8);
      memcpy(&EciParms.eci_userid,       UserID,       8);
      memcpy(&EciParms.eci_password,     PassWd,       8);
      memcpy(&EciParms.eci_system_name,  servers[Idx].SystemName, 8);
      EciParms.eci_commarea                      = CommArea;
      EciParms.eci_commarea_length               = ECI_SYNC_SIZE;
      EciParms.eci_extend_mode                   = ECI_NO_EXTEND;
      EciParms.eci_luw_token                     = ECI_LUW_NEW;
      EciParms.eci_timeout                       = 30;


      /* Call the chosen server with the given parameters - Note the time */

      TimeBefore = CurTimeInMilliSecs();
      rc = CICS_ExternalCall (&EciParms);


      /* If ASYNC, we need to wait for a reply. We could now go and do some
         other processing in the meantime and check the event when we want to,
         but for the purposes of this sample, just wait for it to complete */

      if (AsyncCalls == CCLTRUE) {
         if (Debug == CCLTRUE) {
            CCLLOG2("ECI call %d to server %s return code %d",
                    i, servers[Idx].SystemName, rc);
         }
         if (WaitOnAnEvent( &Callback[AIdx] ) != CCLTRUE) {
            CCLLOG("Failed to wait for call to complete - aborting thread");
            AbortThread();
            EndMyThread();
            return;
         }
         if (ResetAnEvent( &Callback[AIdx] ) != CCLTRUE) {
            CCLLOG("Failed to reset for callback event - aborting thread");
            AbortThread();
            EndMyThread();
            return;
         }


         /* If ASYNC, we need to get the reply, whereas SYNC returns the
              COMMAREA as it completes. Set up an ECI parameter block for the
              getreply call */

         memset(&EciParms, 0, sizeof(ECI_PARMS));

         EciParms.eci_version                           = ECI_VERSION_1A;
         EciParms.eci_call_type                         = ECI_GET_SPECIFIC_REPLY;
         EciParms.eci_message_qualifier                 = (short) AIdx;
         memcpy(&EciParms.eci_userid,       UserID,       8);
         memcpy(&EciParms.eci_password,     PassWd,       8);
         memcpy(&EciParms.eci_system_name,  servers[Idx].SystemName, 8);
         EciParms.eci_commarea                          = CommArea;
         EciParms.eci_commarea_length                   = ECI_SYNC_SIZE;
         EciParms.eci_extend_mode                       = ECI_NO_EXTEND;
         EciParms.eci_luw_token                         = ECI_LUW_NEW;
         EciParms.eci_timeout                           = 30;

         rc = CICS_ExternalCall (&EciParms);
         if (Debug == CCLTRUE) {
            CCLLOG2("ECI get reply call %d, return code %d, COMMAREA %s",
                    i, rc, CommArea);
         }
      } else {


         /* If SYNC, we already have the reply */

         if (Debug == CCLTRUE) {
            CCLLOG2("ECI call %d, return code %d, COMMAREA %s",
                    i, rc, CommArea);
         }
      }


      /* Now the call has completed, note the time and calculate the average */

      TimeAfter = CurTimeInMilliSecs();

      if (rc == ECI_NO_ERROR) {
         average[AIdx] = ((average[AIdx] * (i)) + (TimeAfter-TimeBefore)) / (i+1);
      } else {

         Response(rc, EciParms.eci_abend_code, QueryMyThreadId());
         AbortThread();
         EndMyThread();
         return;
      }
   }


   /* Finish the test -If we are the last server thread, start the main thread
        NOTE: since this is a global variable, must protect access from other
        threads with a mutex semaphore */

   if (GetExclusiveAccess( &ProgressLock ) != CCLTRUE) {
      CCLLOG("Failed to get the exclusive access - aborting thread");
      AbortThread();
      EndMyThread();
      return;
   }

   Progress.NumCompleteOK++;

   if (Progress.NumCompleteOK + Progress.NumCompleteError == Progress.TotalThreads) {
      AllFinished = CCLTRUE;
   }

   if (ReleaseExclusiveAccess( &ProgressLock ) != CCLTRUE) {
      CCLLOG("Failed to release the exclusive access - aborting process");
      exit(2); /* Fatal error */
   }

   /* Cleanup objects created */
   if (AsyncCalls == CCLTRUE) {
      DeleteAnEvent( &Callback[AIdx] );
   }

   if (AllFinished == CCLTRUE) {
      /* Inform main thread all threads are done */

      if (FlagAnEvent( &FinishTest) != CCLTRUE) {
         CCLLOG("Failed to inform main thread via FinishThread - aborting process");
         exit(2); /* Fatal error */
      }
   }

   EndMyThread();
}


/* Function: Response

   Purpose:  Display the immediate response code from an ECI call */

void Response(short Rc, cics_char_t *Abend, CclThreadId Tid)
{
   char *p;
   char *abendString;

   switch (Rc) {
   case ECI_NO_ERROR:                p = "ECI_NO_ERROR               ";break;
   case ECI_ERR_INVALID_DATA_LENGTH: p = "ECI_ERR_INVALID_DATA_LENGTH";break;
   case ECI_ERR_INVALID_EXTEND_MODE: p = "ECI_ERR_INVALID_EXTEND_MODE";break;
   case ECI_ERR_NO_CICS:             p = "ECI_ERR_NO_CICS            ";break;
   case ECI_ERR_CICS_DIED:           p = "ECI_ERR_CICS_DIED          ";break;
   case ECI_ERR_REQUEST_TIMEOUT:     p = "ECI_ERR_REQUEST_TIMEOUT    ";break;
   case ECI_ERR_RESPONSE_TIMEOUT:    p = "ECI_ERR_RESPONSE_TIMEOUT   ";break;
   case ECI_ERR_TRANSACTION_ABEND:   p = "ECI_ERR_TRANSACTION_ABEND  ";break;
   case ECI_ERR_LUW_TOKEN:           p = "ECI_ERR_LUW_TOKEN  "        ;break;
   case ECI_ERR_SYSTEM_ERROR:        p = "ECI_ERR_SYSTEM_ERROR       ";break;
   case ECI_ERR_NULL_WIN_HANDLE:     p = "ECI_ERR_NULL_WIN_HANDLE    ";break;
   case ECI_ERR_NULL_MESSAGE_ID:     p = "ECI_ERR_NULL_MESSAGE_ID    ";break;
   case ECI_ERR_THREAD_CREATE_ERROR: p = "ECI_ERR_THREAD_CREATE_ERROR";break;
   case ECI_ERR_INVALID_CALL_TYPE:   p = "ECI_ERR_INVALID_CALL_TYPE  ";break;
   case ECI_ERR_ALREADY_ACTIVE:      p = "ECI_ERR_ALREADY_ACTIVE     ";break;
   case ECI_ERR_RESOURCE_SHORTAGE:   p = "ECI_ERR_RESOURCE_SHORTAGE  ";break;
   case ECI_ERR_NO_SESSIONS:         p = "ECI_ERR_NO_SESSIONS        ";break;
   case ECI_ERR_NULL_SEM_HANDLE:     p = "ECI_ERR_NULL_SEM_HANDLE    ";break;
   case ECI_ERR_INVALID_DATA_AREA:   p = "ECI_ERR_INVALID_DATA_AREA  ";break;
   case ECI_ERR_INVALID_VERSION:     p = "ECI_ERR_INVALID_VERSION    ";break;
   case ECI_ERR_UNKNOWN_SERVER:      p = "ECI_ERR_UNKNOWN_SERVER     ";break;
   case ECI_ERR_CALL_FROM_CALLBACK:  p = "ECI_ERR_CALL_FROM_CALLBACK ";break;
   case ECI_ERR_MORE_SYSTEMS:        p = "ECI_ERR_MORE_SYSTEMS       ";break;
   case ECI_ERR_NO_SYSTEMS:          p = "ECI_ERR_NO_SYSTEMS         ";break;
   case ECI_ERR_SECURITY_ERROR:      p = "ECI_ERR_SECURITY_ERROR     ";break;
   case ECI_ERR_MAX_SYSTEMS:         p = "ECI_ERR_MAX_SYSTEMS        ";break;
   case ECI_ERR_MAX_SESSIONS:        p = "ECI_ERR_MAX_SESSIONS       ";break;
   case ECI_ERR_ROLLEDBACK:          p = "ECI_ERR_ROLLEDBACK         ";break;
   default:
      {
         p = "!!!Unknown!!!";
         if (Tid != 0) {
            printf("%d : Unknown return code : %d\n", Tid, Rc);
         } else {
            printf("Unknown return code : %d\n", Rc);
         }
      }
      break;
   }

   if (Tid != 0) {
      printf("%d : ECI returned: %s\n", Tid, p);
   } else {
      printf("ECI returned: %s\n", p);
   }

   if (Rc != ECI_NO_ERROR) {
      if (Abend==NULL) {
         abendString="NULL";
      } else {
         abendString=Abend;
      }
      if (Tid != 0) {
         printf("%d : Abend code was \"%4.4s\"\n", Tid, abendString);
      } else {
         printf("Abend code was \"%4.4s\"\n", abendString);
      }
   }

}


/* Function: CallBackProc

   Purpose:  For an ASYNC call, notify the waiting thread the ECI
             call has completed */

void CICSEXIT CallBackProc (cics_ushort_t AIdx) {
   if (Debug == CCLTRUE) printf("%d : Callback running for %d\n",
                                QueryMyThreadId(), AIdx);
   if (FlagAnEvent( &Callback[AIdx] ) != CCLTRUE) {
      printf("%d : Failed to post a callback for terminal index %d - aborting process\n", QueryMyThreadId(), AIdx);
      exit(2); /* Fatal error */
   }
   return;
}

void AbortThread(void) {
   CclBool AllFinished = CCLFALSE;

   if (GetExclusiveAccess( &ProgressLock ) != CCLTRUE) {
      printf("%d : Failed to get exclusive access to progress structure on thread abort - aborting process\n", QueryMyThreadId());
      exit(2); /* Fatal error */
   }

   Progress.NumCompleteError++;

   /* Check we didn't just abort the last thread! */
   if (Progress.NumCompleteOK + Progress.NumCompleteError == Progress.TotalThreads) {
      AllFinished = CCLTRUE;
   }

   if (ReleaseExclusiveAccess( &ProgressLock ) != CCLTRUE) {
      printf("%d : Failed to release exclusive access to progress structure on thread abort - aborting process\n", QueryMyThreadId());
      exit(2); /* Fatal error */
   }

   if (AllFinished == CCLTRUE) {
      /* Inform main thread all threads are done */

      if (FlagAnEvent( &FinishTest) != CCLTRUE) {
         printf("%d : Failed to inform main thread of test completion - aborting process\n", QueryMyThreadId());
         exit(2); /* Fatal error */
      }
   }

}


/* Function: StrToupper

   Purpose:  Takes an array of characters and convert them to upper case */

char *StrToupper(char *str)
{
        int i = 0;

        for(i = 0; i < strlen(str); i++)
                if (str[i] >= 'a' && str[i] <= 'z') str[i] = str[i] - 32;

        return str;
}
