/* File Name(s)  : ecib1.c

   Product       : CICS Transaction Gateway

   Description   : This sample shows the basic use of the CICS Client API.
                   It queries the Client daemon for a list of available servers 
                   and then launches a transaction on the server chosen.

   Prerequisites:  This samples requires the CICS Program EC01 to be
                   installed on your CICS server for operation.

 Licensed Materials - Property of IBM  
  
 5724-I81,5725-B65 
  
 (C) Copyright IBM Corp. 1999, 2014 All Rights Reserved.  
  
 US Government Users Restricted Rights - Use, duplication or  
 disclosure restricted by GSA ADP Schedule Contract with  
 IBM Corp.  
    
    Status: Version 9 Release 1 

                   The following code is sample code created by IBM
                   Corporation.  This sample code is not part of any
                   standard IBM product and is provided to you solely for
                   the purpose of assisting you in the development of your
                   applications.  The code is provided 'AS IS', without
                   warranty or condition of any kind.  IBM shall not be
                   liable for any damages arising out of your use of the
                   sample code, even if IBM has been advised of the
                   possibility of such damages. */


/* Include files */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <cics_eci.h>

/* Structures, defines and macro definitions */

#define ECI_SYNC_SIZE 18

#define copyright_notice copyright4cecib1
char version4cecib1[] =
"\nCICS Transaction Gateway"
" Version 9.1.0.4"
" Build Level c910-20231012-1355" ;

char copyright_notice[] =
"\n"
"Licensed Materials - Property of IBM. "
"\n"
"5724-I81,5725-B65 "
"\n"
"(c) Copyright IBM Corp. 1999, 2014 All Rights Reserved. "
"\n"
"US Government Users Restricted Rights - Use, duplication or "
"disclosure restricted by GSA ADP Schedule Contract with "
"IBM Corp."
"\n";

/* Function prototypes */

void Response(short Rc, cics_char_t *Abend);


/*  Main Function */

int main(){

   CICS_EciSystem_t servers[20];             /*Servers from CICS_EciListSystems */
   cics_ushort_t    serverCount    = 20;     /*Length counter for above list    */
   cics_char_t      progid[4 + 1]  = "EC01"; /*Program to call                  */
   cics_char_t      chosenServer[CICS_ECI_SYSTEM_MAX +1]; /*Selected servername */
   ECI_PARMS        EciParms;                /*The ECI Parameter block          */
   char             CommArea[ECI_SYNC_SIZE]; /*COMMAREA for use in ECI calls    */

   char             UserID[9] = "sysad   ";  /*User ID padded to 8 characters   */
   char             PassWd[9] = "sysad   ";  /*Password padded to 8 characters  */
   char             tempstr[255];            /*Used to validate users input     */

   short            rc;                      /*The return code from the ECI call*/
   int              i, choice  =0;
   int              loop       =0;


   /* Display Banner */

   printf("CICS Transaction Gateway Basic ECI V1 Sample\n\n");

   printf("Servers defined in the Client daemon:\n");


   /* Ask the Client daemon for a list of servers it has defined */

   rc = CICS_EciListSystems(NULL,&serverCount,servers);
   if (rc == ECI_NO_ERROR) {

      do {

         /* Display the list and ask for user input to choose one */

         for (i=0; i<serverCount;i++) {
            printf("   %d.%s\n",i+1 , servers[i].SystemName);
         }

         printf("\nChoose server to connect to:\n");
         scanf("%s", tempstr);
         choice = atol(tempstr);


         /* Validate input and redisplay the list if invalid */

         if (choice > 0 && choice < serverCount+1) {
            strcpy(chosenServer,servers[choice-1].SystemName);
            loop=0;
         } else {
            printf("Out of range\n");
            loop=1;
         }
      }while (loop==1);


      /* Set up the ECI Parameter block */

      memset(CommArea, '\0', ECI_SYNC_SIZE);
      memset(&EciParms, 0, sizeof(ECI_PARMS));

      EciParms.eci_version                           = ECI_VERSION_1A;
      EciParms.eci_call_type                         = ECI_SYNC;
      memcpy(&EciParms.eci_program_name, progid,       4);
      memcpy(&EciParms.eci_userid,       UserID,       8);
      memcpy(&EciParms.eci_password,     PassWd,       8);
      memcpy(&EciParms.eci_system_name,  chosenServer, 8);
      EciParms.eci_commarea                          = CommArea;
      EciParms.eci_commarea_length                   = ECI_SYNC_SIZE;
      EciParms.eci_extend_mode                       = ECI_NO_EXTEND;
      EciParms.eci_luw_token                         = ECI_LUW_NEW;
      EciParms.eci_timeout                           = 30;


      /* Call the chosen server with the given parameters */

      rc = CICS_ExternalCall (&EciParms);
      if (rc == ECI_NO_ERROR) {
         /* Ensure COMMAREA displayed is correct length and '\0' terminated */
         char CommAreaToDisplay[ECI_SYNC_SIZE+1] = {0};
         memcpy(&CommAreaToDisplay, CommArea, ECI_SYNC_SIZE);

         printf("Program %s returned with data: %s\n", progid, CommAreaToDisplay);
      } else {
         Response(rc, EciParms.eci_abend_code);
         return 1;
      }

   } else {
      Response(rc, NULL);
      return 1;
   }

   return 0;
}


/* Function: Response

   Purpose:  Display the immediate response code from an ECI call */

void Response(short Rc, cics_char_t *Abend)
{
   char *p;
   char *abendString;

   switch (Rc) {
   case ECI_NO_ERROR:                p = "ECI_NO_ERROR               ";break;
   case ECI_ERR_INVALID_DATA_LENGTH: p = "ECI_ERR_INVALID_DATA_LENGTH";break;
   case ECI_ERR_INVALID_EXTEND_MODE: p = "ECI_ERR_INVALID_EXTEND_MODE";break;
   case ECI_ERR_NO_CICS:             p = "ECI_ERR_NO_CICS            ";break;
   case ECI_ERR_CICS_DIED:           p = "ECI_ERR_CICS_DIED          ";break;
   case ECI_ERR_REQUEST_TIMEOUT:     p = "ECI_ERR_REQUEST_TIMEOUT    ";break;
   case ECI_ERR_RESPONSE_TIMEOUT:    p = "ECI_ERR_RESPONSE_TIMEOUT   ";break;
   case ECI_ERR_TRANSACTION_ABEND:   p = "ECI_ERR_TRANSACTION_ABEND  ";break;
   case ECI_ERR_LUW_TOKEN:           p = "ECI_ERR_EXEC_LUW_TOKEN     ";break;
   case ECI_ERR_SYSTEM_ERROR:        p = "ECI_ERR_SYSTEM_ERROR       ";break;
   case ECI_ERR_NULL_WIN_HANDLE:     p = "ECI_ERR_NULL_WIN_HANDLE    ";break;
   case ECI_ERR_NULL_MESSAGE_ID:     p = "ECI_ERR_NULL_MESSAGE_ID    ";break;
   case ECI_ERR_THREAD_CREATE_ERROR: p = "ECI_ERR_THREAD_CREATE_ERROR";break;
   case ECI_ERR_INVALID_CALL_TYPE:   p = "ECI_ERR_INVALID_CALL_TYPE  ";break;
   case ECI_ERR_ALREADY_ACTIVE:      p = "ECI_ERR_ALREADY_ACTIVE     ";break;
   case ECI_ERR_RESOURCE_SHORTAGE:   p = "ECI_ERR_RESOURCE_SHORTAGE  ";break;
   case ECI_ERR_NO_SESSIONS:         p = "ECI_ERR_NO_SESSIONS        ";break;
   case ECI_ERR_NULL_SEM_HANDLE:     p = "ECI_ERR_NULL_SEM_HANDLE    ";break;
   case ECI_ERR_INVALID_DATA_AREA:   p = "ECI_ERR_INVALID_DATA_AREA  ";break;
   case ECI_ERR_INVALID_VERSION:     p = "ECI_ERR_INVALID_VERSION    ";break;
   case ECI_ERR_UNKNOWN_SERVER:      p = "ECI_ERR_UNKNOWN_SERVER     ";break;
   case ECI_ERR_CALL_FROM_CALLBACK:  p = "ECI_ERR_CALL_FROM_CALLBACK ";break;
   case ECI_ERR_MORE_SYSTEMS:        p = "ECI_ERR_MORE_SYSTEMS       ";break;
   case ECI_ERR_NO_SYSTEMS:          p = "ECI_ERR_NO_SYSTEMS         ";break;
   case ECI_ERR_SECURITY_ERROR:      p = "ECI_ERR_SECURITY_ERROR     ";break;
   case ECI_ERR_MAX_SYSTEMS:         p = "ECI_ERR_MAX_SYSTEMS        ";break;
   case ECI_ERR_MAX_SESSIONS:        p = "ECI_ERR_MAX_SESSIONS       ";break;
   case ECI_ERR_ROLLEDBACK:          p = "ECI_ERR_ROLLEDBACK         ";break;
   default:
      {
         p = "!!!Unknown!!!";
         printf("Unknown return code : %d\n", Rc);
      }
      break;
   }

   printf("ECI returned: %s\n", p);
   if (Rc != ECI_NO_ERROR) { 
      if (Abend==NULL) {
         abendString="NULL";
      } else {
         abendString=Abend;
      }
      printf("Abend code was \"%4.4s\"\n", abendString);
   }

}
