/*
   MODULE NAME        ctgclient.h

   DESCRIPTIVE NAME   CICS Transaction Gateway ECI and ESI Version 2.3
                      Application Programming Interface

   Service level: V9.1.0.4  
 Licensed Materials - Property of IBM  
  
 "Restricted Materials of IBM"  
  
 5724-I81,5725-B65,5655-Y20 
  
 (C) Copyright IBM Corp. 2010, 2014  
  
 US Government Users Restricted Rights - Use, duplication or  
 disclosure restricted by GSA ADP Schedule Contract with  
 IBM Corp.  

   Status: Version 9 Release 1 

   NOTES :-

   This header file is provided with the CICS Transaction Gateway. These
   products are available for a number of different operating environments
   and to ensure the correct sections of this header are included, one of
   the following constants must be defined prior to inclusion:

     CICS_W32  -  if building a Windows application
     CICS_AIX  -  if building an AIX application
     CICS_SOL  -  if building a Solaris application
     CICS_LNX  -  if building a Linux application
     CICS_HPIT and CICS_HPUX -
                  if building an HPUX application on Itanium
*/
/**
* @file
* Defines functions and typedefs for connecting to a Gateway daemon
* and controlling API tracing.
*/

#ifndef _CTGCLIENT_H
#define _CTGCLIENT_H

/* Include stddef.h for size_t type definition */
#include <stddef.h>

/* Ensure that a platform definition has been defined */
#if !(defined(CICS_W32) || defined(CICS_WNT) || defined(CICS_AIX) || \
        defined(CICS_SOL) || defined(CICS_HPUX) || defined(CICS_LNX) )
  #error Compiler symbol CICS_xxx is not defined (where XXX is one of W32, AIX, SOL, HPUX or LNX)
#endif

#if defined (CICS_W32) || defined(CICS_WNT)
  #define CICSCALL __cdecl
#else
  #define CICSCALL
#endif
/*********************************************************************/
/*             CICS TG ECI Version 2 General constants               */
/*********************************************************************/
/** CICS TG ECI and ESI V2 API version identifier. This is the version
 *  of the API. An application can query the API version at run time to check
 *  that it has been deployed with a compatible CICS TG API library.
 */
#define CTG_API_VERSION           "2.3.0.0"

/** CICS TG ECI and ESI V2 DLL version identifier. This is the
 *  version of the DLL. An application can query the DLL version at run time
 *  and record the version for maintenance or problem determination.
 */
#define CTG_DLL_VERSION           "9.1.0.4"

/** The maximum length of a string returned by CTG_getRcString
 */
#define CTG_MAX_RCSTRING           40

/** Null Gateway daemon token  */
#define CTG_NULL_GWTOK             NULL



/*********************************************************************/
/*        CICS TG ECI and ESI Version 2 Connection functions         */
/*********************************************************************/
/**
 * CTG_ConnToken_t represents a connection to a specific
 * Gateway daemon.
 */
typedef struct _CTG_ConnToken_t * CTG_ConnToken_t;

/**
 * Establish a remote connection to a Gateway daemon. This function
 * takes a character pointer for the hostname, an integer for the
 * target port number and a pointer to a Gateway token and an integer
 * for the connection timeout. It establishes a connection to a Gateway
 * daemon Client protocol handler using the specified hostname and port
 * number. If the return code is successful the Gateway token
 * represents the connection to the specified Gateway daemon, this
 * Gateway token is required as an input parameter for the other API
 * calls.
 *
 * This API supports CICS TG remote mode operation. The CICS TG
 * "local" protocol for local mode operation is not supported.
 *
 * The environment variables CTG_APPLID and CTG_APPLIDQUALIFIER can be
 * used to set the APPLID and APPLID qualifier of the application. The
 * application APPLID and APPLID qualifier can be used for transaction
 * tracking in the Gateway daemon and in CICS.
 *
 * @param[in] address The character string containing the
 * hostname or IP address of the target Gateway daemon.
 * @param[in] port An int for the port of the target Gateway
 * daemon.
 * @param[out] gwTokPtr Pointer to the new Gateway token.
 * @param[in] connTimeout The connection timeout in seconds,
 * enter 0 for no timeout
 * @returns An integer value identifies whether the call was successful or not.
 * \n Possible values include
 * \li #CTG_OK
 * \li #CTG_ERR_MALLOCFAIL
 * \li #CTG_ERR_BADPORT
 * \li #CTG_ERR_CONNECTFAILED
 * \li #CTG_ERR_CONNECTTIMEOUT
 * \li #CTG_ERR_NULLGWTOKPTR
 * \li #CTG_ERR_NULLPARM
 * \li #CTG_ERR_LOSTGWCON
 * \li #CTG_ERR_BADHOST
 * \li #CTG_ERR_NULLADDRESS
 * \li #CTG_ERR_GATEWAY_BACK_LEVEL
 * \li #CTG_ERR_APPLIDTOOLONG
 * \li #CTG_ERR_SYSTEM_ERROR
 */
int CICSCALL CTG_openRemoteGatewayConnection(char * address,
                                             int port,
                                             CTG_ConnToken_t * gwTokPtr,
                                             int connTimeout);
/**
 * Establish a remote connection to a Gateway daemon. This function
 * takes a character pointer for the hostname, an integer for the
 * target port number and a pointer to a Gateway token, an integer
 * for the connection timeout and a null terminated string for both
 * the APPLID and APPLID qualifier. It establishes a connection to a Gateway
 * daemon Client protocol handler using the specified hostname and port
 * number. If the return code is successful the Gateway token
 * represents the connection to the specified Gateway daemon, this
 * Gateway token is required as an input parameter for the other API
 * calls.
 *
 * This API supports CICS TG remote mode operation. The CICS TG
 * "local" protocol for local mode operation is not supported.
 *
 * If the environment variables CTG_APPLID or CTG_APPLIDQUALIFIER are
 * set, their values override the values of the applid and applidQualifier
 * parameters. The application APPLID and APPLID qualifier can be used for
 * transaction tracking in the Gateway daemon and in CICS.
 *
 * @param[in] address The character string containing the
 * hostname or IP address of the target Gateway daemon.
 * @param[in] port An int for the port of the target Gateway
 * daemon.
 * @param[out] gwTokPtr Pointer to the new Gateway token.
 * @param[in] connTimeout The connection timeout in seconds,
 * enter 0 for no timeout
 * @param[in] applid A null terminated character string containing
 * the client APPLID (maximum 8 characters). This parameter is ignored
 * if the CTG_APPLID environment variable is set.
 * @param[in] applidQualifier A null terminated character string
 * containing the client APPLID qualifier (maximum 8 characters). This
 * parameter is ignored if the CTG_APPLIDQUALIFIER environment variable
 * is set.
 * @returns An integer value identifies whether the call was successful or not.
 * \n Possible values include
 * \li #CTG_OK
 * \li #CTG_ERR_MALLOCFAIL
 * \li #CTG_ERR_BADPORT
 * \li #CTG_ERR_CONNECTFAILED
 * \li #CTG_ERR_CONNECTTIMEOUT
 * \li #CTG_ERR_NULLGWTOKPTR
 * \li #CTG_ERR_NULLPARM
 * \li #CTG_ERR_LOSTGWCON
 * \li #CTG_ERR_BADHOST
 * \li #CTG_ERR_NULLADDRESS
 * \li #CTG_ERR_GATEWAY_BACK_LEVEL
 * \li #CTG_ERR_APPLIDTOOLONG
 * \li #CTG_ERR_SYSTEM_ERROR
 */
int CICSCALL CTG_openRemoteGatewayConnectionApplid(char * address,
                                             int port,
                                             CTG_ConnToken_t * gwTokPtr,
                                             int connTimeout,
                                             char * applid,
                                             char * applidQualifier);

/**
 * Closes the connection to the Gateway daemon associated with the
 * specified Gateway token.  Upon successful completion, resources
 * associated with the Gateway daemon connection will be released.
 *
 * When the Gateway token is closed, synchronous requests that are in
 * progress can complete and replies for outstanding asynchronous
 * requests can be retrieved using any of the get reply calls. No new
 * #ECI_SYNC, #ECI_ASYNC or ESI calls can be issued after the close
 * process has started.
 *
 * When a Gateway token is closing, and all outstanding replies have
 * been retrieved, any outstanding #ECI_GET_REPLY_WAIT calls end with
 * #CTG_ERR_GWTOK_CLOSED.
 *
 * This function waits until all synchronous requests are complete
 * and all replies have been retrieved.
 *
 * @param[in] gwTokPtr The reference to the open Gateway connection
 * @returns An integer value identifies whether the call was successful or not.
 * \n Possible values include
 * \li #CTG_OK
 * \li #CTG_ERR_MALLOCFAIL
 * \li #CTG_ERR_NULLGWTOK
 * \li #CTG_ERR_BADGWTOK
 * \li #CTG_ERR_NULLGWTOKPTR
 * \li #CTG_ERR_SYSTEM_ERROR
 */
int CICSCALL CTG_closeGatewayConnection(CTG_ConnToken_t * gwTokPtr);

/**
 * Forcefully closes the connection to the Gateway daemon associated with the
 * specified Gateway token, optionally allowing time for in progress requests
 * to complete and replies for asynchronous requests to be retrieved. Upon
 * successful completion, resources associated with the Gateway daemon
 * connection will be released.
 *
 * The closeTimeout parameter specifies the maximum time, in seconds, to allow
 * for in progress requests to complete and replies for asynchronous requests
 * to be retrieved. During this time, requests that are in progress can
 * complete and replies for asynchronous requests can be retrieved using any of
 * the get reply calls. No further #ECI_SYNC, #ECI_ASYNC or ESI calls can be
 * issued after the close process has started. The timeout must be between zero
 * and 3600 seconds. If a timeout of zero is specified, the connection is
 * closed immediately.
 *
 * Once the close timeout period has elapsed or all requests have completed and
 * all asynchronous replies have been retrieved, the connection to the Gateway
 * daemon is closed. Any outstanding #ECI_GET_REPLY_WAIT and
 * #ECI_GET_SPECIFIC_REPLY_WAIT calls end with #CTG_ERR_GWTOK_CLOSED.
 *
 * @param[in] gwTokPtr The reference to the open Gateway connection
 * @param[in] closeTimeout The time, in seconds, to allow for in progress
 * requests to complete and replies for asynchronous requests to be retrieved.
 * @returns An integer value that identifies whether the call was successful.
 * \n Possible values include
 * \li #CTG_OK
 * \li #CTG_ERR_MALLOCFAIL
 * \li #CTG_ERR_NULLGWTOK
 * \li #CTG_ERR_BADGWTOK
 * \li #CTG_ERR_NULLGWTOKPTR
 * \li #CTG_ERR_INVALIDTIMEOUTPARM
 * \li #CTG_ERR_SYSTEM_ERROR
 */
int CICSCALL CTG_purgeGatewayConnection(CTG_ConnToken_t * gwTokPtr,
                                        int closeTimeout);

/**
 * Attempts to free all resources held by the API, including open
 * Gateway connections. This function is provided to be called in
 * the event of a severe application error, enabling some form
 * of controlled shutdown even if all Gateway tokens have been
 * lost.
 *
 * Any outstanding #ECI_GET_REPLY_WAIT and #ECI_GET_SPECIFIC_REPLY_WAIT
 * calls end with #CTG_ERR_GWTOK_CLOSED. No new calls can be issued on
 * any Gateway token after the close process has started.
 *
 * @returns An integer value identifies whether the call was successful or not.
 * \n Possible values include
 * \li #CTG_OK
 * \li #CTG_ERR_MALLOCFAIL
 * \li #CTG_ERR_SYSTEM_ERROR
 */
int CICSCALL CTG_closeAllGatewayConnections();


/*********************************************************************/
/*         CICS TG ECI and ESI Version 2 Utility functions           */
/*********************************************************************/

/**
 * Get the current trace level. Takes a pointer to an int
 * variable, and sets that variable to the current API trace
 * level.
 *
 * @param[out] traceStatePtr output value, a pointer to an int
 * variable that is set with API trace status.
 * \n Valid values are defined in ctgclient.h they are:
 * \li #CTG_TRACE_LEVEL0 = Disable all trace
 * \li #CTG_TRACE_LEVEL1 = Exception trace
 * \li #CTG_TRACE_LEVEL2 = Event trace
 * \li #CTG_TRACE_LEVEL3 = Entry/exit trace
 * \li #CTG_TRACE_LEVEL4 = Debug trace
 *
 * @returns An integer value identifies whether the call was successful or not.
 * \n Possible values include
 * \li #CTG_OK
 * \li #CTG_ERR_MALLOCFAIL
 * \li #CTG_ERR_NULLPTR
 */
int CICSCALL CTG_getAPITraceLevel(int * traceStatePtr);

/**
 * Set the ECI and ESI version 2 API trace level. Takes an int
 * variable representing the desired trace level and sets the API trace
 * level accordingly. The default trace destination is stderr,
 * but this can be overridden by the API function
 * CTG_setAPITraceFile().
 *
 * The API trace level can also be specified by setting the environment
 * variable CTG_CLIENT_TRACE_LEVEL to the trace level required. Valid
 * values are 0 to 4. The environment variable must be set before the
 * application is started. Use of this function overrides any value
 * obtained from the environment variable.
 *
 * @param[in] traceState an input value, an int variable set to
 *            the required trace level.
 * \n Valid values are defined in ctgclient.h they are:
 * \li #CTG_TRACE_LEVEL0 = Disable all trace
 * \li #CTG_TRACE_LEVEL1 = Exception trace
 * \li #CTG_TRACE_LEVEL2 = Event trace
 * \li #CTG_TRACE_LEVEL3 = Entry/exit trace
 * \li #CTG_TRACE_LEVEL4 = Debug trace

 * @returns An integer value identifies whether the call was successful or not.
 * \n Possible values include
 * \li #CTG_OK
 * \li #CTG_ERR_MALLOCFAIL
 * \li #CTG_ERR_TRACELEVEL
 */
int CICSCALL CTG_setAPITraceLevel(int traceState);

/**
 * Takes a character pointer to a null terminated string
 * representing the desired trace file destination. If the
 * specified file already exists, trace data is appended. If the
 * API is unable to open the specified file for writing,
 * then the trace destination remains unchanged. Specifying a
 * null pointer causes the trace destination to revert to the
 * default destination of stderr.
 *
 * The trace file can also be specified by setting the environment
 * variable CTG_CLIENT_TRACE_FILE to a filename. Any path/filename
 * that is valid for the operating system is an acceptable value.
 * The environment variable must be set before the application is
 * started. Use of this function overrides any value obtained from
 * the environment variable.
 *
 * @param[in] traceFileNamePtr char pointer to a null terminated
 * string representing the desired trace destination,
 * path+filename.
 * @returns An integer value identifies whether the call was successful
 * or not.
 * \n Possible values include
 * \li #CTG_OK
 * \li #CTG_ERR_MALLOCFAIL
 * \li #CTG_ERR_TRACEFILE
 */
int CICSCALL CTG_setAPITraceFile(char * traceFileNamePtr);

/**
 * Sets the maximum length of data that is displayed in ECI and ESI
 * version 2 API trace. The function takes a size_t parameter which
 * specifies the maximum amount of data to trace.
 *
 * The trace data length can also be specified by setting the
 * environment variable CTG_CLIENT_DATA_LENGTH to a positive numeric
 * value. The environment variable must be set before the application
 * is started. Use of this function overrides any value obtained from
 * the environment variable.
 *
 * \note The IBM service organization may request that this
 * function is used to aid problem determination.
 *
 * @param[in] dataLength Maximum length of data to display in trace.
 */
void CICSCALL CTG_setAPITraceDataLength(size_t dataLength);

/**
 * Sets the offset into data that is displayed in ECI and ESI version
 * 2 API trace. The function takes a size_t parameter which specifies
 * the offset into the data.
 *
 * The trace data offset can also be specified by setting the
 * environment variable CTG_CLIENT_DATA_OFFSET to a positive numeric
 * value. The environment variable must be set before the application
 * is started. Use of this function overrides any value obtained from
 * the environment variable.
 *
 * \note The IBM service organization may request that this
 * function is used to aid problem determination.
 *
 * @param[in] dataOffset Offset into data to begin tracing.
 */
void CICSCALL CTG_setAPITraceDataOffset(size_t dataOffset);

/**
 * Writes CICS TG internal debugging information
 * to the trace destination. The debug information is written
 * irrespective of trace settings.
 * \note The IBM service organization may request that this
 * function is used to aid problem determination.
 *
 * @returns An integer value identifies whether the call was successful
 * or not.
 * \n Possible values include
 * \li #CTG_OK
 * \li #CTG_ERR_MALLOCFAIL
 */
int CICSCALL CTG_dumpState();

/**
 * Gets the string representation of the return code.
 *
 * @param[in] returnCode an integer value for naming.
 * @param[out] rcString a string identifier naming the return
 *       code.
 * @returns a pointer to rcString.
 */
char * CICSCALL CTG_getRcString(int returnCode, char * rcString);

/**
 * Gets the API version. This can be used to determine the capabilities of
 * the API:
 * \li Version 2.0 is provided with CICS TG V7.2 and supports synchronous ECI
 * requests with a COMMAREA.
 * \li Version 2.1 is provided with CICS TG V8.0 and introduces support for
 * synchronous ECI requests with a channel.
 * \li Version 2.2 is provided with CICS TG V8.1 and introduces support for
 * password phrases and synchronous ESI verify and change requests.
 * \li Version 2.3 is provided with CICS TG V9.0 and introduces support for
 * asynchronous ECI requests with a COMMAREA or a channel.
 *
 * @param[out] *apiVersPtr a pointer to the api version string. The returned
 * string (returned in the pointer) is owned by the API, and should
 * not be freed by the application.
 * @returns An integer value identifies whether the call was successful
 * or not.
 * \n Possible values include
 * \li #CTG_OK
 * \li #CTG_ERR_NULLPTR
 * \li #CTG_ERR_MALLOCFAIL
 */
int CICSCALL CTG_getAPIVersion(char ** apiVersPtr);

/**
 * Gets the DLL version. This will change when the build level changes.
 *
 * @param[out] *dllVersPtr a pointer to the dll version string. The returned
 * string (returned in the pointer) is owned by the API, and should
 * not be freed by the application.
 * @returns An integer value identifies whether the call was successful
 * or not.
 * \n Possible values include
 * \li #CTG_OK
 * \li #CTG_ERR_NULLPTR
 * \li #CTG_ERR_MALLOCFAIL
 */
int CICSCALL CTG_getDLLVersion(char ** dllVersPtr);

/**
 * Gets the APPLID.
 * @param[in] gwTok The reference to the open Gateway connection
 * @param[out] *applid a pointer to the APPLID string. The returned
 * string (returned in the pointer) is owned by the API, and should
 * not be freed by the application.
 * @returns An integer value identifies whether the call was successful
 * or not.
 * \n Possible values include
 * \li #CTG_OK
 * \li #CTG_ERR_NULLPTR
 * \li #CTG_ERR_MALLOCFAIL
 * \li #CTG_ERR_NULLGWTOK
 * \li #CTG_ERR_SYSTEM_ERROR
 */
int CICSCALL CTG_getApplid(CTG_ConnToken_t gwTok, char ** applid);

/**
 * Gets the APPLID qualifier.
 *
 * @param[in] gwTok The reference to the open Gateway connection
 * @param[out] *applidQualifier a pointer to the APPLID qualifier string.
 * The returned string (returned in the pointer) is owned by the API,
 * and should not be freed by the application.
 * @returns An integer value identifies whether the call was successful
 * or not.
 * \n Possible values include
 * \li #CTG_OK
 * \li #CTG_ERR_NULLPTR
 * \li #CTG_ERR_MALLOCFAIL
 * \li #CTG_ERR_NULLGWTOK
 * \li #CTG_ERR_SYSTEM_ERROR
 */
int CICSCALL CTG_getApplidQualifier(CTG_ConnToken_t gwTok, char ** applidQualifier);
/*****************************************************************************/
/**
 * The data for a server as returned from calling CTG_listSystems.
 * CTG_listSystem_t is used to hold the name and
 * description of a server definition. An array of
 * CTG_listSystem_t is used when
 * CTG_listSystems is called.
 */
typedef struct
{
   /* Length of SystemName */
   #define CTG_LIST_SYSTEM_LENGTH        8
   /**
   * SystemName identifies the name of the server. When SystemName
   * is shorter than #CTG_LIST_SYSTEM_LENGTH SystemName is
   * padded with nulls to a length of #CTG_LIST_SYSTEM_LENGTH + 1.
   */
   char SystemName[CTG_LIST_SYSTEM_LENGTH+1];

   /* Length of the Description */
   #define CTG_LIST_DESCRIPTION_LENGTH  60
   /**
   *  Description is the description of the server identified
   *  by SystemName.  When the description is shorter than
   *  #CTG_LIST_DESCRIPTION_LENGTH  characters, it is padded
   *  with nulls to a length of #CTG_LIST_DESCRIPTION_LENGTH + 1.
   */
   char Description[CTG_LIST_DESCRIPTION_LENGTH+1];
} CTG_listSystem_t;

/*****************************************************************************/
/**
 *  CTG_listSystems returns a list of all the systems known to the
 *  Gateway, including IPIC and logical CICS servers (only available on
 *  z/OS). Logical servers can be identified by a prefix of (LS) in the
 *  Description field. Although the list of systems are known to the
 *  Gateway there is no guarantee that a communications link exists or
 *  that any of the servers is available to process requests.
 *
 * @param[in] gwTok The reference to the open Gateway connection.
 * @param Systems Used for input and output, on input the maximum number of
 * systems allowed for in List, on return the actual number of systems found.
 * @param[out] List on return, the reference to an array of
 *  CTG_ListSystem_t structures containing the system
 *  names and descriptions.
 * @returns An integer value identifies whether the call was successful or not.
 * \n Possible values include
 * \li #CTG_OK
 * \li #CTG_ERR_MALLOCFAIL
 * \li #CTG_ERR_NULLGWTOKPTR
 * \li #CTG_ERR_NULLPARM
 * \li #CTG_ERR_LOSTGWCON
 * \li #CTG_ERR_NULLECIPOINTER
 * \li #CTG_ERR_GATEWAY_CLOSED
 * \li #CTG_ERR_WORK_WAS_REFUSED
 * \li #CTG_ERR_GATEWAY_EXCEPTION
 * \li #CTG_ERR_INVALID_REQUEST_TYPE
 * \li #CTG_ERR_MORE_SYSTEMS
 * \li #CTG_ERR_NO_SYSTEMS
 * \li #CTG_ERR_SYSTEM_ERROR
*/
int CICSCALL CTG_listSystems(CTG_ConnToken_t gwTok,
                             unsigned short *Systems,
                             CTG_listSystem_t *List);

/*********************************************************************/
/*      CICS TG ECI and ESI Version 2 Return Code constants          */
/*********************************************************************/
/**
 * The API call completed successfully.
 */
#define CTG_OK                       0
/**
 * \deprecated No longer used.
 */
#define CTG_ERR_INVALID_DATA_LENGTH  -1
/**
 * An internal system error has occurred. If the problem persists,
 * collect application and Gateway daemon trace and contact your IBM
 * service representative.
 */
#define CTG_ERR_SYSTEM_ERROR         -9
/**
 * There are more systems to return than were requested. The total
 * number of systems is returned in the Systems parameter.
 */
#define CTG_ERR_MORE_SYSTEMS         -25
/**
 * There are no servers known to the Gateway daemon.
 */
#define CTG_ERR_NO_SYSTEMS           -26
/**
 * The Gateway token is NULL.
 */
#define CTG_ERR_NULLGWTOK            -100
/**
 * The Gateway token is not valid. This might be because the
 * connection to the Gateway daemon has been lost and the token deleted.
 */
#define CTG_ERR_BADGWTOK             -101
/**
 * The connection to the Gateway daemon has been lost.
 */
#define CTG_ERR_LOSTGWCON            -102
/**
 * One of the parameters is NULL.
 */
#define CTG_ERR_NULLPARM             -103
/**
 * The trace level is not valid.
 */
#define CTG_ERR_TRACELEVEL           -104
/**
 * The port number is not valid. Port numbers must be in the
 * range 1 through 65535.
 */
#define CTG_ERR_BADPORT              -107
/**
 * The Gateway token pointer is NULL.
 */
#define CTG_ERR_NULLGWTOKPTR         -108
/**
 * The pointer is NULL.
 */
#define CTG_ERR_NULLPTR              -109
/**
 * The API was unable to allocate sufficient memory to complete the
 * operation. If the problem persists, contact your IBM service
 * representative.
 */
#define CTG_ERR_MALLOCFAIL           -111
/**
 * \deprecated No longer used.
 */
#define CTG_ERR_BADGWTOKLIST         -115
/**
 * Failed to establish a connection with the Gateway daemon.
 */
#define CTG_ERR_CONNECTFAILED        -118
/**
 * \deprecated No longer used.
 */
#define CTG_ERR_ICONV                -120
/**
 * \deprecated No longer used.
 */
#define CTG_ERR_LOCKFAIL             -121
/**
 * \deprecated No longer used.
 */
#define CTG_ERR_COMMSFAIL            -122
/**
 * \deprecated No longer used.
 */
#define CTG_ERR_PIDMISMATCH          -123
/**
 * \deprecated No longer used.
 */
#define CTG_ERR_TIDMISMATCH          -124
/**
 * Unable to create a trace file at the specified location.
 */
#define CTG_ERR_TRACEFILE            -130
/**
 * The hostname did not resolve to a valid IP address.
 */
#define CTG_ERR_BADHOST              -132
/**
 * The hostname is NULL.
 */
#define CTG_ERR_NULLADDRESS          -133
/**
 * The connection timed out.
 */
#define CTG_ERR_CONNECTTIMEOUT       -134
/**
 * The ECI parameter block pointer is NULL.
 */
#define CTG_ERR_NULLECIPOINTER       -137
/**
 * The timeout parameter is not in the range 0 through 3600.
 */
#define CTG_ERR_INVALIDTIMEOUTPARM   -138
/**
 * The connection failed because the Applid or Applid Qualifier
 * is greater than 8 characters in length.
 */
#define  CTG_ERR_APPLIDTOOLONG       -139
/**
 * The Gateway token has been closed and no new work can be sent to the Gateway
 * daemon.
 */
#define CTG_ERR_GWTOK_CLOSED         -140
/**
 * \deprecated No longer used.
 */
#define CTG_ERR_UNKNOWN_REQUEST_TYPE 0xF002
/**
 * The connection to the Gateway daemon is closed. This occurs when the
 * Gateway daemon is shut down.
 */
#define CTG_ERR_GATEWAY_CLOSED       0xF004
/**
 * The Gateway daemon refused the request because no worker
 * threads were available or the Gateway daemon is closing.
 */
#define CTG_ERR_WORK_WAS_REFUSED     0xF005
/**
 * An internal exception occurred in the Gateway daemon. If the problem
 * persists, collect Gateway daemon trace and contact your IBM service
 * representative.
 */
#define CTG_ERR_GATEWAY_EXCEPTION    0xF006
/**
 * \deprecated No longer used.
 */
#define CTG_ERR_NOT_AUTHORIZED       0xF007
/**
 * The connection failed because the Gateway daemon is at an earlier product
 * release than the Client application.
 */
#define CTG_ERR_GATEWAY_BACK_LEVEL   0xF00A
/**
 * The protocol handler for the port that the Client application is connected to
 * does not accept the request type sent. This can occur if your application
 * connects to the Gateway daemon statistics port instead of the TCP/IP port.
 */
#define CTG_ERR_INVALID_REQUEST_TYPE 0xF00B

/*********************************************************************/
/*        CICS TG ECI and ESI Version 2 Trace constants              */
/*********************************************************************/
/**
 * Trace level 0 disables all tracing.
 */
#define CTG_TRACE_LEVEL0              0
/**
 * Trace level 1 includes exception trace points only.
 */
#define CTG_TRACE_LEVEL1              1
/**
 * Trace level 2 includes event trace points + lower trace
 * levels.
 */
#define CTG_TRACE_LEVEL2              2
/**
 * Trace level 3 includes entry/exit trace points + lower trace
 * levels.
 */
#define CTG_TRACE_LEVEL3              3
/**
 * Trace level 4 includes debug trace points + lower trace
 * levels.
 */
#define CTG_TRACE_LEVEL4              4


#endif /* ifndef _CTGCLIENT_H */

/* Doxygen index page  */

/*! \mainpage CICS TG ECI and ESI Version 2 for C Overview
 * <p>
 * This documentation is for the CICS Transaction Gateway ECI and ESI
 * Version 2 for C.  This application programming interface (API)
 * provides support for C-language ECI and ESI applications to access
 * CICS servers from multiplatform machines remote to the Gateway daemon.
 * Such an application must connect to a Gateway daemon over TCP/IP
 * before flowing an ECI or ESI request to the target CICS server.
 * The API is supported on the following platforms: <ul>
 * <li> AIX </li>
 * <li> HP-UX on Itanium </li>
 * <li> Linux on Intel </li>
 * <li> Linux on POWER </li>
 * <li> Linux on zSeries </li>
 * <li> Solaris </li>
 * <li> Windows </li> </ul>
 * </p><p>
 * The API is not supported on z/OS but connectivity from a remote ECI
 * or ESI application to CICS TG for z/OS is supported.
 * </p><p>
 * ECI and ESI version 2 C sample programs are provided with the CICS
 * TG product, the samples are installed in the following directories: <ul>
 * <li> <b> &lt;install_path&gt;/samples/c/eci_v2 </b> </li>
 * <li> <b> &lt;install_path&gt;/samples/c/esi_v2 </b> </li> </ul>
 * </p><p>
 * \htmlonly
 * <b>Helpful links</b>
 * <ul>
 * <li><a href="globals_func.html">API Functions</a></li>
 * </ul>
 * \endhtmlonly
 * </p><p>
 * The types and functions which make up this API are defined in header files
 * ctgclient.h, ctgclient_eci.h, ctgclient_esi.h and eci_channel.h.
 * </p><p>
 * When running on AIX, HP-UX, Solaris or Linux, the ECI and ESI version 2
 * API disables signal handling for the SIGPIPE signal.
 * </p><p>
 * When an application written for ECI and ESI version 2 is ready for
 * deployment the appropriate CICS TG DLL or shared object file must be made
 * available on the remote system. The CICS TG file for deployment with the
 * application varies with operating system and application architecture:</p>
 * \htmlonly
 * <table role="presentation">
 * <tr><td> AIX, 32-bit and 64-bit </td>
 *     <td> &lt;install_path&gt;/lib/libctgclient.a </td></tr>
 * <tr><td> Windows, 32-bit </td>
 *     <td> &lt;install_path&gt;\lib\ctgclient.dll </td></tr>
 * <tr><td> Windows, 64-bit </td>
 *     <td> &lt;install_path&gt;\lib64\ctgclient.dll </td></tr>
 * <tr><td> Other supported platforms, 32-bit </td>
 *     <td> &lt;install_path&gt;/lib/libctgclient.so </td></tr>
 * <tr><td> Other supported platforms, 64-bit </td>
 *     <td> &lt;install_path&gt;/lib64/libctgclient.so </td></tr>
 * </table>
 * \endhtmlonly
 * </p><p>
 */
