/*
   MODULE NAME        ctgclient_esi.h

   DESCRIPTIVE NAME   CICS Transaction Gateway ECI and ESI Version 2.3
                      Application Programming Interface, ESI Parts

   Service level: V9.1.0.4  
 Licensed Materials - Property of IBM  
  
 "Restricted Materials of IBM"  
  
 5724-I81,5725-B65,5655-Y20 
  
 (C) Copyright IBM Corp. 2011, 2014  
  
 US Government Users Restricted Rights - Use, duplication or  
 disclosure restricted by GSA ADP Schedule Contract with  
 IBM Corp.  

   Status: Version 9 Release 1 

   NOTES :-

   This header file is provided with the CICS Transaction Gateway. These
   products are available for a number of different operating environments
   and to ensure the correct sections of this header are included, one of
   the following constants must be defined prior to inclusion:

     CICS_W32  -  if building a Windows application
     CICS_AIX  -  if building an AIX application
     CICS_SOL  -  if building a Solaris application
     CICS_LNX  -  if building a Linux application
     CICS_HPIT and CICS_HPUX -
                  if building an HPUX application on Itanium
*/
/**
* @file
* Defines functions and typedefs for sending External Security Interface
* (ESI) requests to a Gateway daemon.
*/
#ifndef _CTG_ESI_H
#define _CTG_ESI_H

#include <time.h>
#include "ctgclient.h"

#if defined (CICS_W32) || defined(CICS_WNT)
  #define CICSCALL __cdecl
#else
  #define CICSCALL
#endif

#if !defined(_LP64) && !defined(__LP64__) && !defined(_WIN64) && !defined(__64BIT__)
#if defined(CICS_AIX)
#pragma options align=packed
#elif defined(CICS_HPUX)
#pragma pack 1
#else
#pragma pack(1)
#endif
#endif

/**
 * The ESI_DATETIME structure contains the component parts for a date
 * and time returned as part of the meta-data returned from an ESI request.
 *
 * The function CTG_ESI_convertTime can be used to convert this structure
 * into a tm structure.
 */
typedef struct {
   int year;
   int month;
   int day;
   int hour;
   int minute;
   int second;
   int hundredths;
} ESI_DATETIME;

/**
 * The ESI_DETAILS structure contains the meta-data returned from an ESI
 * call.
 */
typedef struct {
   /**
    * \brief The date and time when the user ID was last verified.
    */
   ESI_DATETIME lastVerifiedTime;
   /**
    * \brief The date and time when the user ID was last accessed.
    */
   ESI_DATETIME lastAccessedTime;
   /**
    * \brief The data and time when the current password will expire.
    */
   ESI_DATETIME expiryTime;
   /**
    * \brief The number of invalid password verification attempts for the user
    * ID since the last successful password verification. This value is zero on
    * a successful invocation of the CTG_ESI_changePassword function.
    */
   int          invalidCount;
} ESI_DETAILS;

/**
 * The maximum length of a server name.
 */
#define ESI_SERVER_LENGTH    8

/**
 * Sends an ESI request to the Gateway to verify a User ID and password or
 * password phrase and then waits for the reply. The reply is returned in the
 * ESI_DETAILS structure.
 *
 * @param[in] gwTok The reference to the open Gateway connection.
 * @param[in] userid The User ID to verify in the CICS server.
 * @param[in] password The password or password phrase for the supplied User ID.
 * @param[in] server The name of the CICS server the request should be
 * sent to. The server used by the Gateway daemon may be changed depending on
 * the high availability configuration.
 * @param[out] esiDetailsPtr A pointer to an ESI_DETAILS structure.
 *
 * @returns An integer value identifies whether the call was successful or not.
 * \n Possible values include
 * \li #CTG_OK
 * \li #CTG_ERR_LOCKFAIL
 * \li #CTG_ERR_MALLOCFAIL
 * \li #CTG_ERR_BADGWTOKLIST
 * \li #CTG_ERR_NULLGWTOKPTR
 * \li #CTG_ERR_LOSTGWCON
 * \li #CTG_ERR_GATEWAY_CLOSED
 * \li #CTG_ERR_WORK_WAS_REFUSED
 * \li #CTG_ERR_GATEWAY_EXCEPTION
 * \li #CTG_ERR_INVALID_REQUEST_TYPE
 *
 * Other possible values are all of the documented ESI return codes.
 *
*/
int CICSCALL CTG_ESI_verifyPassword(CTG_ConnToken_t gwTok,
                                    const char* userid,
                                    const char* password,
                                    const char* server,
                                    ESI_DETAILS* esiDetailsPtr);

/**
 * Sends an ESI request to the Gateway to change the password or password
 * phrase for the given user ID and then waits for the reply. The reply is
 * returned in the ESI_DETAILS structure.
 *
 * @param[in] gwTok The reference to the open Gateway connection.
 * @param[in] userid The User ID to verify in the CICS server.
 * @param[in] oldPassword The current password or password phrase for the
 * supplied User ID.
 * @param[in] newPassword The new password or password phrase for the supplied
 * User ID.
 * @param[in] server The name of the CICS server the request should be
 * sent to. The server used by the Gateway daemon may be changed depending on
 * the high availability configuration.
 * @param[out] esiDetailsPtr A pointer to an ESI_DETAILS structure.
 *
 * @returns An integer value identifies whether the call was successful or not.
 * \n Possible values include
 * \li #CTG_OK
 * \li #CTG_ERR_LOCKFAIL
 * \li #CTG_ERR_MALLOCFAIL
 * \li #CTG_ERR_BADGWTOKLIST
 * \li #CTG_ERR_NULLGWTOKPTR
 * \li #CTG_ERR_LOSTGWCON
 * \li #CTG_ERR_GATEWAY_CLOSED
 * \li #CTG_ERR_WORK_WAS_REFUSED
 * \li #CTG_ERR_GATEWAY_EXCEPTION
 * \li #CTG_ERR_INVALID_REQUEST_TYPE
 *
 * Other possible values are all of the documented ESI return codes.
 *
*/
int CICSCALL CTG_ESI_changePassword(CTG_ConnToken_t gwTok,
                                    const char* userid,
                                    const char* oldPassword,
                                    const char* newPassword,
                                    const char* server,
                                    ESI_DETAILS* esiDetailsPtr);


/**
 * Converts a supplied ESI_DATETIME structure into a tm structure.
 * The tm structure is defined by time.h
 *
 * @param[in] esiTime A pointer to an ESI_Details structure.
 * @param[out] result A pointer to a tm structure.
 *
 * @returns The pointer supplied in result.
 */
struct tm* CICSCALL CTG_ESI_convertTime(const ESI_DATETIME* esiTime,
                                        struct tm* result);

/**
 * No error.
 */
#define ESI_NO_ERROR 0
/**
 * userid is rejected as invalid by the target CICS server.
 */
#define ESI_ERR_USERID_INVALID -300
/**
 * userid is NULL or an empty string.
 */
#define ESI_ERR_NULL_USERID -301
/**
 * password is not the current password or password phrase for userid
 * on the target CICS server.
 */
#define ESI_ERR_PASSWORD_INVALID -302
/**
 * password is NULL or an empty string.
 */
#define ESI_ERR_NULL_PASSWORD -303
/**
 * The protocol used to communicate with the target CICS server does
 * not support Password Expiry Management.
 */
#define ESI_ERR_PEM_NOT_SUPPORTED -304
/**
 * Password Expiry Management has been disabled on the target CICS
 * server.
 */
#define ESI_ERR_PEM_NOT_ACTIVE -305
/**
 * The password for userid on the target CICS server has expired.
 */
#define ESI_ERR_PASSWORD_EXPIRED -306
/**
 * An error was detected by the External Security Manager.
 * This could be because the user ID has been revoked or the CICS server is not
 * configured to communicate with the External Security Manager.
 */
#define ESI_ERR_SECURITY_ERROR -307
/**
 * There are insufficient resources to communicate with the target
 * CICS server; details might be available in the CICS TG error log
 * files.
 */
#define ESI_ERR_RESOURCE_SHORTAGE -308
/**
 * There are insufficient communication resources to communicate with
 * the target CICS server.
 */
#define ESI_ERR_NO_SESSIONS -309
/**
 * Communication with the target CICS server could not be established.
 */
#define ESI_ERR_NO_CICS -310
/**
 * Communication with the target CICS server was interrupted during
 * the request.
 */
#define ESI_ERR_CICS_DIED -311
/**
 * The target CICS server is unknown to the Gateway daemon.
 */
#define ESI_ERR_UNKNOWN_SERVER -312
/**
 * An internal system error has occurred. If the problem persists,
 * collect application and Gateway daemon trace and contact your IBM
 * service representative.
 */
#define ESI_ERR_SYSTEM_ERROR -313
/**
 * The value of the old password was NULL or an empty string.
 */
#define ESI_ERR_NULL_OLD_PASSWORD ESI_ERR_NULL_PASSWORD
/**
 * The value of the new password was NULL or an empty string.
 */
#define ESI_ERR_NULL_NEW_PASSWORD -314
/**
 * The new password does not conform to the standards defined for the
 * External Security Manager of the target CICS server.
 */
#define ESI_ERR_PASSWORD_REJECTED -315
/**
 * The CICS TG has reached its limit on the maximum number of CICS
 * servers.
 */
#define ESI_ERR_MAX_SYSTEMS          -316
/**
 * There are insufficient communication resources to complete the
 * request.
 */
#define ESI_ERR_MAX_SESSIONS         -317


#if !defined(_LP64) && !defined(__LP64__) && !defined(_WIN64) && !defined(__64BIT__)
#if defined(CICS_AIX)
#pragma options align=reset
#elif defined(CICS_HPUX) && defined(__cplusplus)
#pragma pack
#elif defined(CICS_HPUX) && defined(CICS_HPIT)
#pragma pack
#elif defined(CICS_HPUX)
#pragma HP_ALIGN HPUX_NATURAL
#else
#pragma pack()
#endif
#endif

#endif /* _CTG_ESI_H */
