/*
   MODULE NAME        eci_channel.h

   DESCRIPTIVE NAME   CICS Transaction Gateway ECI and ESI Version 2.3
                      Application Programming Interface, Channels and
                      Containers Parts

   Service level: V9.1.0.4  
 Licensed Materials - Property of IBM  
  
 "Restricted Materials of IBM"  
  
 5724-I81,5725-B65,5655-Y20 
  
 (C) Copyright IBM Corp. 2010, 2014  
  
 US Government Users Restricted Rights - Use, duplication or  
 disclosure restricted by GSA ADP Schedule Contract with  
 IBM Corp.  
   
   Status: Version 9 Release 1 

   NOTES :-

   This header file is provided with the CICS Transaction Gateway. These
   products are available for a number of different operating environments
   and to ensure the correct sections of this header are included, one of
   the following constants must be defined prior to inclusion:

     CICS_W32  -  if building a Windows application
     CICS_AIX  -  if building an AIX application
     CICS_SOL  -  if building a Solaris application
     CICS_LNX     if building a Linux application
     CICS_HPIT and CICS_HPUX -
                  if building an HPUX application on Itanium
*/
/**
* @file
* Defines functions and typedefs for creating and using channels and
* containers.
*/

#ifndef _ECI_CHANNEL_H
#define _ECI_CHANNEL_H

#include <stdlib.h>

/* Ensure that a platform definition has been defined */
#if !(defined(CICS_W32) || defined(CICS_WNT) || defined(CICS_AIX) || \
        defined(CICS_SOL) || defined(CICS_HPUX) || defined(CICS_LNX) )
  #error Compiler symbol CICS_xxx is not defined (where XXX is one of W32, AIX, SOL, HPUX or LNX)
#endif

#if defined (CICS_W32) || defined(CICS_WNT)
  #define CICSCALL __cdecl
#else
  #define CICSCALL
#endif


#if !defined(_LP64) && !defined(__LP64__) && !defined(_WIN64) && !defined(__64BIT__)
#if defined(CICS_AIX)
#pragma options align=packed
#elif defined(CICS_HPUX)
#pragma pack 1
#else
#pragma pack(1)
#endif
#endif



/** Represents a channel object */
typedef struct _ECI_Channel_t* ECI_ChannelToken_t;

/** Null channel token */
#define ECI_NULL_CHANNELTOKEN (ECI_ChannelToken_t) NULL


/** Container type enumeration */
typedef enum {

   /** \brief The container holds binary data */
   ECI_BIT  = 0x01,

   /** \brief The container holds character data */
   ECI_CHAR = 0x02

} ECI_CONTAINER_TYPE;


/** Container information structure */
typedef struct {

   #define ECI_CONTAINERNAME_LENGTH    16
   /** \brief The container name */
   char name[ECI_CONTAINERNAME_LENGTH + 1];

   /** \brief The container type */
   ECI_CONTAINER_TYPE type;

   /** \brief The code character set identifier (CCSID) of the
    *  container data                                         
    */
   int ccsid;

   /** \brief The length of the container data */
   size_t dataLength;

} ECI_CONTAINER_INFO;


#define ECI_CHANNELNAME_LENGTH   16


/** \brief Creates a new channel with the specified name and provides
 * the caller with a channel token for use with other channel related
 * functions.
 *
 * Channel names are up to #ECI_CHANNELNAME_LENGTH characters in length
 * and are case sensitive. Allowable characters are:
 * \li Uppercase characters A-Z
 * \li Lowercase characters a-z
 * \li Digits 0-9
 * \li Special characters \< \> $ @ # / % & ? ! : | " = ' ; . - _
 *
 * It is not possible to modify a channel name after creation.
 *
 * #ECI_CHAR data always has an associated coded character set
 * identifier (CCSID) which describes the encoding of the data.
 *
 * When a channel is created, a default CCSID is set based upon the
 * encoding of the current locale of the application. If no mapping
 * exists for the current locale, the CCSID for US-ASCII is used.
 *
 * The default CCSID for a channel is replaced by the CCSID of the
 * first container of type #ECI_CHAR to be added to the channel.
 *
 * Adding subsequent containers of any type or CCSID has no effect
 * upon the channel CCSID and it cannot be changed explicitly.
 *
 * @param[in] name The name for the new channel.
 * @param[out] tokPtr Pointer to a channel token. If the call is
 * successful, tokPtr is modified to point at the new channel token.
 *
 * @returns An integer value identifies whether the call was successful or not.
 * \n Possible values include
 * \li #ECI_NO_ERROR
 * \li #ECI_ERR_INVALID_NAME
 * \li #ECI_ERR_NULLCHANNELPTR
 * \li #ECI_ERR_MEMORY_SHORTAGE
 * \li #ECI_ERR_CHANNELLOCKFAIL
 * \li #ECI_ERR_LOCKFAIL
 *
 * @sa ECI_createContainer()
 */
int CICSCALL ECI_createChannel(const char* name,
                               ECI_ChannelToken_t* tokPtr);

/** \brief Deletes the channel represented by the specified channel
 * token and associated containers.
 *
 * Following a successful call, any copy of the channel token or
 * ECI_CONTAINER_INFO variable representing an associated container
 * is invalidated.
 *
 * @param[in,out] tokPtr Pointer to a channel token. If the call is
 * successful, tokPtr is modified to point to #ECI_NULL_CHANNELTOKEN.
 *
 * @returns An integer value identifies whether the call was successful or not.
 * \n Possible values include
 * \li #ECI_NO_ERROR
 * \li #ECI_ERR_NULLCHANNELPTR
 * \li #ECI_ERR_INVALID_CHANNEL
 * \li #ECI_ERR_LOCKFAIL
 */
int CICSCALL ECI_deleteChannel(ECI_ChannelToken_t* tokPtr);


/** \brief Deletes all existing channels and associated containers.
 *
 * This function can be used by the API program under normal circumstances,
 * or in the event of a severe error, enabling some form of controlled
 * shutdown when references to channel tokens might have been lost.
 *
 * @returns An integer value identifies whether the call was successful or not.
 * \n Possible values include
 * \li #ECI_NO_ERROR
 * \li #ECI_ERR_LOCKFAIL
 */
int CICSCALL ECI_deleteAllChannels();


/** \brief Gets the name of the channel represented by the specified channel
 * token.
 *
 * Following a successful call, nameBuffer contains a null terminated string
 * up to #ECI_CHANNELNAME_LENGTH bytes in length (excluding null terminator).
 * The channel name is set using the ECI_createChannel() function.
 *
 * @param[in] chanTok A channel token representing the channel from which
 * to obtain the name.
 * @param[out] nameBuffer A pointer to a character buffer at least
 * #ECI_CHANNELNAME_LENGTH+1 in length, into which the channel name is copied.
 *
 * @returns An integer value identifies whether the call was successful or not.
 * \n Possible values include
 * \li #ECI_NO_ERROR
 * \li #ECI_ERR_INVALID_CHANNEL
 * \li #ECI_ERR_NULLPTR
 * \li #ECI_ERR_LOCKFAIL
 */
int CICSCALL ECI_getChannelName(ECI_ChannelToken_t chanTok,
                                char* nameBuffer);


/** \brief Gets the first container from the channel represented by
 * the specified channel token.
 *
 * Functions ECI_getFirstContainer() and ECI_getNextContainer() enable you to
 * iterate over a set of containers within a channel.
 *
 * Following a successful call, the referenced struct ECI_CONTAINER_INFO
 * contains details of the first container in the channel.
 *
 * @param[in] chanTok A channel token representing the channel from which
 * to obtain the container.
 * @param[out] infoPtr A pointer to an ECI_CONTAINER_INFO struct, into
 * which information regarding the first container is copied.
 *
 * @returns An integer value identifies whether the call was successful or not.
 * \n Possible values include
 * \li #ECI_NO_ERROR
 * \li #ECI_ERR_INVALID_CHANNEL
 * \li #ECI_ERR_ENDOFCONTAINERS
 * \li #ECI_ERR_NULLINFOPTR
 * \li #ECI_ERR_LOCKFAIL
 *
 * @sa ECI_getNextContainer()
 */
int CICSCALL ECI_getFirstContainer(ECI_ChannelToken_t chanTok,
                                   ECI_CONTAINER_INFO* infoPtr);


/** \brief Gets the next container from the channel represented by
 * the specified channel token.
 *
 * Functions ECI_getFirstContainer() and ECI_getNextContainer() enable you to
 * iterate over a set of containers within a channel.
 *
 * Following a successful call, the referenced struct ECI_CONTAINER_INFO
 * contains details of the next container in the channel.
 *
 * If this call follows a call to the ECI_getFirstContainer()
 * function for the same channel, the container information is
 * set with details of the second container in the channel, if
 * available.
 *
 * If this call follows a call to the ECI_getNextContainer()
 * function for the same channel, the container information is
 * set with details of the next available container in the channel.
 *
 * If this call does not follow a call to the ECI_getFirstContainer() or
 * ECI_getNextContainer() function for the same channel, the container
 * information is set with details of the first container in the channel.
 *
 * @param[in] chanTok A channel token representing the channel from which
 * to obtain the container.
 * @param[out] infoPtr A pointer to an ECI_CONTAINER_INFO struct, into
 * which information regarding the next container is copied.
 *
 * @returns An integer value identifies whether the call was successful or not.
 * \n Possible values include
 * \li #ECI_NO_ERROR
 * \li #ECI_ERR_INVALID_CHANNEL
 * \li #ECI_ERR_ENDOFCONTAINERS
 * \li #ECI_ERR_NULLINFOPTR
 * \li #ECI_ERR_LOCKFAIL
 *
 * @sa ECI_getFirstContainer()
 */
int CICSCALL ECI_getNextContainer(ECI_ChannelToken_t chanTok,
                                  ECI_CONTAINER_INFO* infoPtr);


/** \brief Creates a new container in the channel associated with the
 * specified channel token.
 *
 * Container names are up to #ECI_CONTAINERNAME_LENGTH characters in
 * length and are case sensitive. Allowable characters are:
 * \li Uppercase characters A-Z
 * \li Lowercase characters a-z
 * \li Digits 0-9
 * \li Special characters \< \> $ @ # / % & ? ! : | " = ' ; . - _
 *
 * It is not possible to modify a container name or type after
 * creation.
 *
 * #ECI_CHAR data always has an associated coded character set
 * identifier (CCSID) which describes the encoding of the data.
 *
 * When a container is created of type #ECI_CHAR with a CCSID of zero,
 * the container CCSID is set to the channel CCSID.
 *
 * The default CCSID for a channel is replaced by the CCSID of the
 * first container of type #ECI_CHAR to be added to the channel.
 *
 * Adding subsequent containers of any type or CCSID has no effect
 * upon the channel CCSID and it cannot be changed explicitly.
 *
 * @param[in] chanTok A channel token representing the channel to
 * which the new container is added.
 * @param[in] name A pointer to a null terminated character string,
 * up to #ECI_CHANNELNAME_LENGTH bytes in length (excluding null
 * terminator), to be used as the name for the new container.
 * @param[in] type The type of container to be created:
 * \li #ECI_BIT
 * \li #ECI_CHAR
 * @param[in] ccsid A CCSID value that specifies the encoding of the data when
 * the container type is #ECI_CHAR. The CCSID is ignored when the container type
 * is #ECI_BIT. Set to zero when container type is #ECI_CHAR to use the channel
 * CCSID.
 * @param[in] data A pointer to the data to be copied into the new
 * container.
 * @param[in] dataLength The number of bytes of data to be copied into the
 * new container.
 *
 * @returns An integer value identifies whether the call was successful or not.
 * \n Possible values include
 * \li #ECI_NO_ERROR
 * \li #ECI_ERR_INVALID_CHANNEL
 * \li #ECI_ERR_INVALID_NAME
 * \li #ECI_ERR_INVALID_TYPE
 * \li #ECI_ERR_CONTAINER_EXISTS
 * \li #ECI_ERR_INVALID_DATA_LENGTH
 * \li #ECI_ERR_MEMORY_SHORTAGE
 * \li #ECI_ERR_LOCKFAIL
 *
 * @sa ECI_createChannel()
 */
int CICSCALL ECI_createContainer(ECI_ChannelToken_t chanTok,
                                 const char* name,
                                 ECI_CONTAINER_TYPE type,
                                 const int ccsid,
                                 const void* data,
                                 size_t dataLength);


/** \brief Deletes the named container from the channel associated
 * with the specified channel token.
 *
 * @param[in] chanTok A channel token representing the channel from which
 * the named container is deleted.
 * @param[in] name A pointer to a null terminated character string,
 * up to #ECI_CHANNELNAME_LENGTH bytes in length (excluding null terminator),
 * representing the name of the container to be deleted.
 *
 * @returns An integer value identifies whether the call was successful or not.
 * \n Possible values include
 * \li #ECI_NO_ERROR
 * \li #ECI_ERR_INVALID_CHANNEL
 * \li #ECI_ERR_INVALID_NAME
 * \li #ECI_ERR_CONTAINER_NOTFOUND
 * \li #ECI_ERR_LOCKFAIL
 */
int CICSCALL ECI_deleteContainer(ECI_ChannelToken_t chanTok,
                                 const char* name);


/** \brief Gets information about the named container in the channel
 * associated with the specified channel token.
 *
 * @param[in] chanTok A channel token representing the channel from which
 * information for the named container is obtained.
 * @param[in] name A pointer to a null terminated character string,
 * up to #ECI_CHANNELNAME_LENGTH bytes in length (excluding null terminator),
 * representing the name of the container.
 * @param[out] infoPtr A pointer to an ECI_CONTAINER_INFO struct, into
 * which information regarding the named container is copied.
 *
 * @returns An integer value identifies whether the call was successful or not.
 * \n Possible values include
 * \li #ECI_NO_ERROR
 * \li #ECI_ERR_INVALID_CHANNEL
 * \li #ECI_ERR_INVALID_NAME
 * \li #ECI_ERR_CONTAINER_NOTFOUND
 * \li #ECI_ERR_NULLINFOPTR
 * \li #ECI_ERR_LOCKFAIL
 */
int CICSCALL ECI_getContainerInfo(ECI_ChannelToken_t chanTok,
                                const char* name,
                                ECI_CONTAINER_INFO* infoPtr);


/** \brief Gets data from the named container in the channel associated
 * with the specified channel token.
 *
 * Data from the specified container can be copied to the buffer in a
 * single call or iteratively. The size of the application buffer must be
 * at least the number of bytes specified in parameter \p length.
 *
 * @param[in] chanTok A channel token representing the channel from which
 * data for the named container is obtained.
 * @param[in] name A pointer to a null terminated character string,
 * up to #ECI_CHANNELNAME_LENGTH bytes in length (excluding null terminator),
 * representing the name of the container.
 * @param[out] buffer A pointer to a data buffer into which the data
 * for the named container is copied.
 * @param[in] length The maximum number of bytes of data to copy into
 * the buffer.
 * @param[in] offset The number of bytes into the named container data from
 * which the specified number of bytes are copied.
 * @param[out] bytesReadPtr A pointer to a size_t variable, into which the
 * number of bytes copied is set. This might be less than or equal to the
 * number of bytes requested if the end of the data available is reached.
 * Set to \p NULL if this information is not required.
 *
 * @returns An integer value identifies whether the call was successful or not.
 * \n Possible values include
 * \li #ECI_NO_ERROR
 * \li #ECI_ERR_INVALID_CHANNEL
 * \li #ECI_ERR_INVALID_NAME
 * \li #ECI_ERR_CONTAINER_NOTFOUND
 * \li #ECI_ERR_NULLPTR
 * \li #ECI_ERR_LOCKFAIL
 */
int CICSCALL ECI_getContainerData(ECI_ChannelToken_t chanTok,
                                  const char* name,
                                  void* buffer,
                                  size_t length,
                                  size_t offset,
                                  size_t* bytesReadPtr);


/** \brief Copies data into the named container in the channel associated
 * with the specified channel token.
 *
 * Data from the specified location is copied to the named container. If
 * the container type is #ECI_CHAR, the data is assumed to be of the same
 * encoding specified on the ECI_createContainer() call.
 *
 * @param[in] chanTok A channel token representing the channel into which
 * data for the named container is set.
 * @param[in] name A pointer to a null terminated character string,
 * up to #ECI_CHANNELNAME_LENGTH bytes in length (excluding null terminator),
 * representing the name of the container.
 * @param[in] data A pointer to the data to be copied into the named
 * container.
 * @param[in] dataLength The number of bytes of data to copy into the
 * named container. If set to zero, the named container is set to empty.
 *
 * @returns An integer value identifies whether the call was successful or not.
 * \n Possible values include
 * \li #ECI_NO_ERROR
 * \li #ECI_ERR_INVALID_CHANNEL
 * \li #ECI_ERR_INVALID_NAME
 * \li #ECI_ERR_CONTAINER_NOTFOUND
 * \li #ECI_ERR_INVALID_DATA_LENGTH
 * \li #ECI_ERR_MEMORY_SHORTAGE
 * \li #ECI_ERR_LOCKFAIL
 */
int CICSCALL ECI_setContainerData(ECI_ChannelToken_t chanTok,
                                  const char* name,
                                  const void* data,
                                  size_t dataLength);


/** No error. */
#define ECI_NO_ERROR                   0

/** Bad data length. */
#define ECI_ERR_INVALID_DATA_LENGTH    -1

/** An internal system error has occurred. */
#define ECI_ERR_SYSTEM_ERROR         -9

/** The channel token is not valid. */
#define ECI_ERR_INVALID_CHANNEL        -50

/** The specified name is not valid. */
#define ECI_ERR_INVALID_NAME           -51

/** The channel token pointer is NULL. */
#define ECI_ERR_NULLCHANNELPTR         -52

/** Unable to create a lock for the channel. */
#define ECI_ERR_CHANNELLOCKFAIL        -53

/** There are no more containers in the channel. */
#define ECI_ERR_ENDOFCONTAINERS        -54

/** The ECI_CONTAINER_INFO pointer is NULL. */
#define ECI_ERR_NULLINFOPTR            -55

/** The container type is not valid. */
#define ECI_ERR_INVALID_TYPE           -56

/** The specified container does not exist in the channel. */
#define ECI_ERR_CONTAINER_NOTFOUND     -58

/** A container with the same name already exists in the channel. */
#define ECI_ERR_CONTAINER_EXISTS       -59

/** Both a COMMAREA and a channel are specified. */
#define ECI_ERR_COMMAREA_AND_CHANNEL   -60

/** A pointer parameter was NULL. */
#define ECI_ERR_NULLPTR                -109

/** The API was unable to allocate sufficient memory to complete the
 * operation. */
#define ECI_ERR_MEMORY_SHORTAGE        -111

/** Internal error during lock creation. */
#define ECI_ERR_LOCKFAIL               -121



#if !defined(_LP64) && !defined(__LP64__) && !defined(_WIN64) && !defined(__64BIT__)
#if defined(CICS_AIX)
#pragma options align=reset
#elif defined(CICS_HPUX) && defined(__cplusplus)
#pragma pack
#elif defined(CICS_HPUX) && defined(CICS_HPIT)
#pragma pack
#elif defined(CICS_HPUX)
#pragma HP_ALIGN HPUX_NATURAL
#else
#pragma pack()
#endif
#endif


#endif /* _ECI_CHANNEL_H */
