/* 
   File Name(s)  : ctgecib1.c

   Product       : CICS Transaction Gateway

   Description   : This sample shows the basic usage of the remote support
                   in the CICS Transaction Gateway ECI API version 2. The
                   program queries the CICS TG for a list of available
                   servers and then calls a program on the chosen server.

   Prerequisites:  This sample requires the CICS program EC01 to be
                   installed on your CICS server for operation.

 Licensed Materials - Property of IBM  
  
 5724-I81,5725-B65,5655-Y20 
  
 (C) Copyright IBM Corp. 2008, 2014 All Rights Reserved.  
  
 US Government Users Restricted Rights - Use, duplication or  
 disclosure restricted by GSA ADP Schedule Contract with  
 IBM Corp.  

    Status: Version 9 Release 1 

    The following code is sample code created by IBM
    Corporation.  This sample code is not part of any
    standard IBM product and is provided to you solely for
    the purpose of assisting you in the development of your
    applications.  The code is provided 'AS IS', without
    warranty or condition of any kind.  IBM shall not be
    liable for any damages arising out of your use of the
    sample code, even if IBM has been advised of the
    possibility of such damages.
*/


/* Include files */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/* Remote ECI support functions */
#include <ctgclient_eci.h>

/* Structures, defines and macro definitions */
#define COMMAREA_SIZE   18    /* Size of commarea to pass to EC01 program         */
#define MAX_ATTEMPTS    3     /* Maximum number of times to attempt CICS call     */
#define MAX_SERVERS     40    /* Maximum number of servers to return from CICS TG */

/* Return codes used by the program */
#define RC_OK     CTG_OK         /* Return code for no error            */
#define RC_QUIT   1              /* Return code if the user enters 'q'  */
#define RC_FAIL   2              /* Return code for a failure condition */

#ifndef NULL
#define NULL   0
#endif

#define copyright_notice copyright4cctgecib1
char version4cctgecib1[] =
"\nCICS Transaction Gateway"
" Version 9.1.0.4"
" Build Level c910-20231012-1355" ;

char copyright_notice[] =
"\n"
"Licensed Materials - Property of IBM. "
"\n"
"5724-I81,5725-B65,5655-Y20 "
"\n"
"(c) Copyright IBM Corp. 2008, 2014 All Rights Reserved. "
"\n"
"US Government Users Restricted Rights - Use, duplication or "
"disclosure restricted by GSA ADP Schedule Contract with "
"IBM Corp."
"\n";

/* Function prototypes */
int openGatewayConnection(CTG_ConnToken_t* gatewayTokenOut, char* hostname, int port, int connTimeout);
int callProgram(CTG_ConnToken_t gatewayToken, char* serverName);
int closeGatewayConnection(CTG_ConnToken_t* gatewayTokenPtr);
int selectServer(CTG_ConnToken_t gatewayToken, char* serverName);
void inputUserCredentials(char** userId, char** password);
void displayRc(int rc, char* abendCode);


/*
   Function: main

   Purpose:  Main function for the program. It performs the following steps:
               1. Parses commandline arguments
               2. Opens a connection to the CICS TG
               3. Lists the servers defined on the CICS TG and prompts the user to select a server
               4. Calls the EC01 program on the chosen server
               5. Closes the CICS TG connection
*/
int main(int argc, char** argv) {

   CTG_ConnToken_t gatewayToken;             /* Gateway token                       */
   char* hostname = "localhost";             /* Host name of the CICS TG            */
   int port = 2006;                          /* Port of the CICS TG TCP/IP handler  */
   char serverName[ECI_SYSTEM_NAME_LENGTH];  /* CICS server to call EC01 on         */
   int rc = RC_OK;                           /* Return code                         */
   int connTimeout = 60;
   
   /* Display banner and usage */
   printf("CICS Transaction Gateway Basic ECI V2 Sample\n\n");
   printf("Usage: ctgecib1 [host name] [port number]\n\n");

   /* 1. Parse commandline arguments */
   /* Check for host name parameter */
   if (argc > 1) {
      hostname = argv[1];
   }

   /* Check for port parameter */
   if (argc > 2) {
      port = atoi(argv[2]);
      if ((port < 1) || (port > 65536)) {
         printf("Invalid port specified, defaulting to 2006\n\n");
         port = 2006;
      }
   }

   /* 2. Open connection to CICS TG */
   rc = openGatewayConnection(&gatewayToken, hostname, port, connTimeout);
   if (rc == RC_OK) {

      /* 3. List servers defined on the CICS TG and prompt the user to select a server */
      rc = selectServer(gatewayToken, serverName);
      if (rc == RC_OK) {

         /* 4. Call EC01 program */
         rc = callProgram(gatewayToken, serverName);
      }

      /* 5. Close the CICS TG connection */
      if (closeGatewayConnection(&gatewayToken) != RC_OK) {
         rc = RC_FAIL;
      }
   }

   /* Display results of program */
   if (rc == RC_FAIL) {
      printf("ctgecib1 encountered errors during execution\n");
   } else {
      printf("ctgecib1 completed successfully\n");
   }

   return rc;
}



/*
   Function: openGatewayConnection

   Purpose:  Opens a connection to the CICS TG using the host name and port provided, and if
             successful returns the gateway token in gatewayTokenOut.
*/
int openGatewayConnection(CTG_ConnToken_t* gatewayTokenOut, char* hostname, int port, int connTimeout) {

   int rc = CTG_OK;    /* Return code                                                    */

   /* Open CICS TG connection */
   rc = CTG_openRemoteGatewayConnection(hostname, port, gatewayTokenOut,connTimeout);

   if (rc != CTG_OK) {
      printf("Failed to connect to CICS TG on %s using port %d\n", hostname, port);
      displayRc(rc, NULL);
      rc = RC_FAIL;
   } else {
      printf("Connected to CICS TG on %s using port %d\n\n", hostname, port);
   }

   return rc;
}


/*
   Function: callProgram

   Purpose:  Calls the EC01 CICS program on the given CICS server.
*/
int callProgram(CTG_ConnToken_t gatewayToken, char* serverName) {
   
   CTG_ECI_PARMS eciParms;                             /* ECI parameter block                   */
   char commarea[COMMAREA_SIZE];                       /* Commarea                              */
   char program[ECI_PROGRAM_NAME_LENGTH + 1] = "EC01"; /* Program to call, 8 characters maximum */
   char transId[ECI_TRANSID_LENGTH + 1] = "";          /* Transaction ID, 4 characters maximum  */
   char tpn[ECI_TPN_LENGTH + 1] = "";                  /* TPN name, 4 characters maximum        */
   char* userId = NULL;                                /* CICS user ID                          */
   char* passWd = NULL;                                /* CICS password                         */
   int attempts = 0;                                   /* Number of attempts                    */
   int rc = CTG_OK;                                    /* Return code                           */

   /* Prepare the ECI parameter block */
   memset(&eciParms, 0, sizeof(CTG_ECI_PARMS));
   memset(commarea, 0, COMMAREA_SIZE);

   eciParms.eci_version = ECI_VERSION_2A;          /* ECI version 2A includes support for       */
                                                   /* password phrases                          */
   eciParms.eci_call_type = ECI_SYNC;              /* Synchronous ECI call                      */

   eciParms.eci_commarea = commarea;               /* Commarea                                  */
   eciParms.eci_commarea_length = COMMAREA_SIZE;   /* Actual length of the commarea             */
   eciParms.commarea_outbound_length = 0;          /* Length of commarea to send to CICS TG     */
                                                   /* - zero to send the entire commarea        */
   eciParms.commarea_inbound_length = 0;           /* Length of commarea to return from CICS TG */
                                                   /* - zero to return the entire commarea      */

   eciParms.eci_extend_mode = ECI_NO_EXTEND;       /* Non-extended call                         */
   eciParms.eci_luw_token = ECI_LUW_NEW;           /* Zero for a new LUW or non-extended call   */
   eciParms.eci_timeout = 0;                       /* Timeout in seconds - zero for no timeout  */

   /* The maximum length of program and server names is 8 characters. */
   /* They are not null terminated.                                   */
   memcpy(&eciParms.eci_program_name, program, strlen(program));
   memcpy(&eciParms.eci_system_name, serverName, ECI_SYSTEM_NAME_LENGTH);

   /* The maximum length of transid and tpn is 4 characters. They are */
   /* not null terminated.                                            */
   memcpy(&eciParms.eci_transid, transId, strlen(transId));
   memcpy(&eciParms.eci_tpn, tpn, strlen(tpn));

   do {

      /* The eci_userid_ptr and eci_password_ptr fields can be set to */
      /* null terminated strings containing the user ID and password. */

      /* The maximum length of a user ID or password depends on the   */
      /* type of CICS server being called and the communications      */
      /* protocol used between CICS TG and CICS.                      */
      eciParms.eci_userid_ptr = userId;
      eciParms.eci_password_ptr = passWd;
  
      /* Execute the ECI request with the given parameters */
      rc = CTG_ECI_Execute(gatewayToken, &eciParms);

      /* After the call, release the memory allocated for user ID  */
      /* and password if they were set.                            */
      if (userId != NULL) {
         free(userId);
         userId = NULL;
      }
      if (passWd != NULL) {
         free(passWd);
         passWd = NULL;
      }

      /* Check for security errors */
      if (rc == ECI_ERR_SECURITY_ERROR) {
         printf("An ECI_ERR_SECURITY_ERROR error occurred\n");

         /* Prompt for user ID and password */
         inputUserCredentials(&userId, &passWd);
         attempts++;
      }

   } while ((rc == ECI_ERR_SECURITY_ERROR) && (attempts < MAX_ATTEMPTS));

   if (rc != CTG_OK) {
      printf("\nCTG_ECI_Execute call failed\n");
      displayRc(rc, eciParms.eci_abend_code);
      rc = RC_FAIL;
   } else {
      /* Display the returned commarea */
      /* Ensure CommArea displayed is correct length and '\0' terminated */
      char commareaToDisplay[COMMAREA_SIZE+1] = {0};
      memcpy(&commareaToDisplay, commarea, COMMAREA_SIZE);

      printf("\nProgram %s returned with data: %s\n\n", program, commareaToDisplay);
   }

   return rc;
}


/*
   Function: closeGatewayConnection

   Purpose:  Closes the connection to the CICS TG.
*/
int closeGatewayConnection(CTG_ConnToken_t* gatewayTokenPtr) {

   int rc = CTG_OK;  /* Return code */

   /* Close connection to CICS TG */
   rc = CTG_closeGatewayConnection(gatewayTokenPtr);

   if (rc != CTG_OK) {
      /* If the CTG_closeGatewayConnection function fails, either communication with the */
      /* CICS TG has been lost, or the gateway token has been corrupted. If communication with  */
      /* the CICS TG has been lost, then the program can probably continue safely. If the       */
      /* gateway token has been corrupted then this indicates a more serious error.             */

      printf("Failed to close connection to CICS Transaction Gateway\n");
      displayRc(rc, NULL);
      rc = RC_FAIL;
   } else {
      printf("Closed connection to CICS Transaction Gateway\n\n");
   }

   return rc;
}


/*
   Function: selectServer

   Purpose:  Queries the CICS TG for a list of available servers, and prompts the user to select a
             server. The name of the chosen server is copied into variable serverName.
*/
int selectServer(CTG_ConnToken_t gatewayToken, char* serverName) {

   CTG_listSystem_t servers[MAX_SERVERS];       /* CICS servers returned from the CICS TG */
   unsigned short serverCount = MAX_SERVERS;    /* Number of CICS servers returned        */
   char inputBuffer[8];                         /* Buffer to receive user input           */
   int choice = 0;                              /* Server choice                          */
   int rc = CTG_OK;                             /* Return code                            */
   int i = 0;                                   /* Loop variable                          */
   

   /* Perform CTG_listSystems call */
   rc = CTG_listSystems(gatewayToken, &serverCount, servers);

   if (rc != CTG_OK) {
      printf("CTG_listSystems call failed\n");
      displayRc(rc, NULL);
      rc = RC_FAIL;
   }

   /* Get server choice from user */
   while ((choice == 0) && (rc == RC_OK)) {

      /* Print out server list. Server names and descriptions */
      /* are null terminated.                                 */
      printf("Servers returned from CICS TG:\n");
      for (i = 0; i < serverCount; i++) {
         printf("   %d. %s - %s\n", i + 1, servers[i].SystemName,
                servers[i].Description);
      }

      /* Get user input */
      printf("\nChoose server to connect to, or q to quit:\n");
      fgets(inputBuffer, sizeof(inputBuffer), stdin);

      /* Check for quit */
      if ((inputBuffer[0] == 'q') || (inputBuffer[0] == 'Q')) {
         rc = RC_QUIT;

      } else {
         /* Validate user input */
         choice = atol(inputBuffer);
         
         if ((choice < 1) || (choice > serverCount)) {
            printf("Out of range\n");
            choice = 0;
         }
      }
   }

   if (rc == RC_OK) {
      /* Copy server name into serverName variable */
      memcpy(serverName, servers[choice - 1].SystemName, CTG_LIST_SYSTEM_LENGTH);
   }

   return rc;
}



/*
   Function: inputUserCredentials

   Purpose:  Prompts the user to enter a user ID and password or password phrase.
*/
void inputUserCredentials(char** userId, char** password) {

   char inputBuffer[128];  /* Buffer to receive user input */

   /* Prompt for user ID */
   memset(inputBuffer, 0, sizeof(inputBuffer));
   printf("Enter your CICS user ID:\n");
   fgets(inputBuffer, sizeof(inputBuffer), stdin);
   inputBuffer[strlen(inputBuffer) - 1] = '\0';
   *userId = strdup(inputBuffer);

   /* Prompt for password or password phrase */
   memset(inputBuffer, 0, sizeof(inputBuffer));
   printf("Enter your CICS password or password phrase:\n");
   fgets(inputBuffer, sizeof(inputBuffer), stdin);
   inputBuffer[strlen(inputBuffer) - 1] = '\0';
   *password = strdup(inputBuffer);
}



/*
   Function: displayRc

   Purpose:  Display the return code and abend code from an API call.
*/
void displayRc(int rc, char* abendCode) {

   char rcString[CTG_MAX_RCSTRING + 1];  /* Return code string */

   /* Get the return code string */
   CTG_getRcString(rc, rcString);

   /* Display return code */
   printf("Return code: %s (%d)\n", rcString, rc);

   if (abendCode != NULL) {
      /* Display abend code */
      printf("Abend code : %.4s\n", abendCode);
   }

   printf("\n");
}
