/* 
   File Name(s)  : ctgecib3.c

   Product       : CICS Transaction Gateway

   Description   : This sample shows the basic usage of the channels and
                   containers support in the CICS Transaction Gateway ECI
                   API version 2. The program queries the CICS TG for a list
                   of available servers and then calls program EC03 on the
                   chosen server. The contents of any containers returned from
                   the program are displayed.

   Prerequisites:  This sample requires a CICS server supporting channels and
                   containers with the supplied CICS program EC03 installed.

 Licensed Materials - Property of IBM  
  
 5724-I81,5725-B65,5655-Y20 
  
 (C) Copyright IBM Corp. 2010, 2014 All Rights Reserved.  
  
 US Government Users Restricted Rights - Use, duplication or  
 disclosure restricted by GSA ADP Schedule Contract with  
 IBM Corp.  

    Status: Version 9 Release 1 

    The following code is sample code created by IBM
    Corporation.  This sample code is not part of any
    standard IBM product and is provided to you solely for
    the purpose of assisting you in the development of your
    applications.  The code is provided 'AS IS', without
    warranty or condition of any kind.  IBM shall not be
    liable for any damages arising out of your use of the
    sample code, even if IBM has been advised of the
    possibility of such damages.
*/


/* Include files */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/* ECIv2 functions */
#include <ctgclient_eci.h>

/* Structures, defines and macro definitions    */

/* Name of channel to pass to EC03 program      */
#define CHANNEL_NAME    "SAMPLECHANNEL"
/* Name of container to pass to EC03 program    */
#define CONTAINER_NAME  "INPUTDATA"
/* Size of the container, defined to match EC03 */
#define MAX_DATA        72
/* Number of times to attempt CICS call         */
#define MAX_ATTEMPTS    3
/* Number of servers to return from CICS TG     */
#define MAX_SERVERS     20

/* Return codes used by the program */
#define RC_OK     0              /* Return code for no error            */
#define RC_QUIT   1              /* Return code if the user enters 'q'  */
#define RC_FAIL   2              /* Return code for a failure condition */

#define copyright_notice copyright4cctgecib3
char version4cctgecib3[] =
"\nCICS Transaction Gateway"
" Version 9.1.0.4"
" Build Level c910-20231012-1355" ;

char copyright_notice[] =
"\n"
"Licensed Materials - Property of IBM. "
"\n"
"5724-I81,5725-B65,5655-Y20 "
"\n"
"(c) Copyright IBM Corp. 2010, 2014 All Rights Reserved. "
"\n"
"US Government Users Restricted Rights - Use, duplication or "
"disclosure restricted by GSA ADP Schedule Contract with "
"IBM Corp."
"\n";

/* Function prototypes */
int openGatewayConnection(CTG_ConnToken_t * gatewayTokenOut, char * hostname,
                          int port, int connTimeout);
int callProgram(CTG_ConnToken_t gatewayToken, char * serverName,
                ECI_ChannelToken_t channel);
int closeGatewayConnection(CTG_ConnToken_t * gatewayTokenPtr);
int selectServer(CTG_ConnToken_t gatewayToken, char * serverName);
int createChannel(ECI_ChannelToken_t * channelTokPtr);
int displayChannelContents(ECI_ChannelToken_t channel);
int deleteChannel(ECI_ChannelToken_t * channelTokPtr);
void inputUserCredentials(char ** userId, char ** password);
void displayRc(int ctgRc);


/*
   Function: main

   Purpose:  Main function for the program. It performs the following steps:
               1. Parses command line arguments
               2. Opens a connection to the CICS TG
               3. Lists the servers defined on the CICS TG and prompts the user
                  to select a server
               4. Creates a channel to pass to the EC03 program
               5. Calls the EC03 program on the chosen server
               6. Displays the containers returned from the EC03 program
               7. Closes the CICS TG connection
*/
int main(int argc, char ** argv) {

   CTG_ConnToken_t gatewayToken;             /* Gateway token                 */
   char * hostname = "localhost";            /* Host name of the CICS TG      */
   int port = 2006;                          /* TCP/IP port of the CICS TG    */
   char serverName[ECI_SYSTEM_NAME_LENGTH];  /* CICS server to call EC03 on   */
   ECI_ChannelToken_t channelToken;          /* Channel token                 */
   int rc = RC_OK;                           /* Return code                   */
   int connTimeout = 60;                     /* Connection timeout            */
   
   /* Display banner and usage */
   printf("CICS Transaction Gateway Basic ECI V2 Sample 3\n\n");
   printf("Usage: ctgecib3 [host name] [port number]\n\n");

   /* 1. Parse command line arguments */
   /* Check for host name parameter */
   if (argc > 1) {
      hostname = argv[1];
   }

   /* Check for port parameter */
   if (argc > 2) {
      port = atoi(argv[2]);
      if ((port < 1) || (port > 65536)) {
         printf("Invalid port specified, defaulting to 2006\n\n");
         port = 2006;
      }
   }

   /* 2. Open connection to CICS TG */
   rc = openGatewayConnection(&gatewayToken, hostname, port, connTimeout);
   if (rc == RC_OK) {

      /* 3. List servers defined on the CICS TG and prompt the user to */
      /*    select a server                                            */
      rc = selectServer(gatewayToken, serverName);
      if (rc == RC_OK) {

         /* 4. Create a channel to pass to the EC03 program */
         rc = createChannel(&channelToken);
         
         if (rc == RC_OK) {
            /* 5. Call EC03 program */
            rc = callProgram(gatewayToken, serverName, channelToken);

            if (rc == RC_OK) {
               /* 6. Display the containers returned from the EC03 program */
               rc = displayChannelContents(channelToken);
            }

            if (deleteChannel(&channelToken) != RC_OK) {
               rc = RC_FAIL;
            }
         }
      }

      /* 7. Close the CICS TG connection */
      if (closeGatewayConnection(&gatewayToken) != RC_OK) {
         rc = RC_FAIL;
      }
   }

   if (rc == RC_FAIL) {
      printf("ctgecib3 encountered errors during execution\n");
   } else {
      printf("ctgecib3 completed successfully\n");
   }

   return rc;
}



/*
   Function: openGatewayConnection

   Purpose:  Opens a connection to the CICS TG using the host name and port
             provided, and if successful returns the gateway token in
             gatewayTokenOut.
*/
int openGatewayConnection(CTG_ConnToken_t * gatewayTokenOut, char * hostname,
                          int port, int connTimeout) {

   int ctgRc = CTG_OK;  /* ECIv2 return code    */
   int rc = RC_OK;      /* Function return code */

   /* Open CICS TG connection */
   ctgRc = CTG_openRemoteGatewayConnection(hostname, port, gatewayTokenOut,
                                           connTimeout);

   if (ctgRc != CTG_OK) {
      printf("Failed to connect to CICS TG on %s using port %d\n", hostname,
             port);
      displayRc(ctgRc);
      rc = RC_FAIL;
   } else {
      printf("Connected to CICS TG on %s using port %d\n\n", hostname, port);
   }

   return rc;
}


/*
   Function: callProgram

   Purpose:  Calls the EC03 CICS program on the given CICS server.
*/
int callProgram(CTG_ConnToken_t gatewayToken, char * serverName,
                ECI_ChannelToken_t channelToken) {
   
   CTG_ECI_PARMS eciParms = {0};                       /* ECI parameter block */
   char program[ECI_PROGRAM_NAME_LENGTH + 1] = "EC03"; /* Program to call     */
   char transId[ECI_TRANSID_LENGTH + 1] = "";          /* Transaction ID      */
   char tpn[ECI_TPN_LENGTH + 1] = "";                  /* TPN name            */
   char * userId = NULL;                               /* CICS user ID        */
   char * passWd = NULL;                               /* CICS password       */
   int attempts = 0;                                   /* Number of attempts  */
   int ctgRc = CTG_OK;                                 /* ECIv2 return code   */
   int rc = RC_OK;                                     /* Function return code*/

   /* Prepare the ECI parameter block */
   memset(&eciParms, 0, sizeof(CTG_ECI_PARMS));

   eciParms.eci_version = ECI_VERSION_2A;          /* ECI version 2A          */
   eciParms.eci_call_type = ECI_SYNC;              /* Synchronous ECI call    */

   eciParms.eci_extend_mode = ECI_NO_EXTEND;       /* Non-extended call       */
   eciParms.eci_luw_token = ECI_LUW_NEW;           /* Zero for a new LUW      */
   eciParms.eci_timeout = 0;                       /* Timeout in seconds      */

   /* The maximum length of program and server names is 8 characters. */
   /* They are not null terminated.                                   */
   memcpy(&eciParms.eci_program_name, program, strlen(program));
   memcpy(&eciParms.eci_system_name, serverName, ECI_SYSTEM_NAME_LENGTH);

   /* The maximum length of transid and tpn is 4 characters. They are */
   /* not null terminated.                                            */
   memcpy(&eciParms.eci_transid, transId, strlen(transId));
   memcpy(&eciParms.eci_tpn, tpn, strlen(tpn));

   /* Set the channel on the request.                                 */
   eciParms.channel = channelToken;

   do {

      /* The eci_userid_ptr and eci_password_ptr fields can be set to */
      /* null terminated strings containing the user ID and password. */

      /* The maximum length of a user ID or password depends on the   */
      /* type of CICS server being called and the communications      */
      /* protocol used between CICS TG and CICS.                      */
      eciParms.eci_userid_ptr = userId;
      eciParms.eci_password_ptr = passWd;
  
      /* Execute the ECI request with the given parameters    */

      /* CTG_ECI_Execute_Channel is used to pass a channel to */
      /* the CICS program.                                    */
      ctgRc = CTG_ECI_Execute_Channel(gatewayToken, &eciParms);

      /* After the call, release the memory allocated for user ID  */
      /* and password if they were set.                            */
      if (userId != NULL) {
         free(userId);
         userId = NULL;
      }
      if (passWd != NULL) {
         free(passWd);
         passWd = NULL;
      }

      /* Check for security errors */
      if (ctgRc == ECI_ERR_SECURITY_ERROR) {
         printf("An ECI_ERR_SECURITY_ERROR error occurred\n");

         /* Prompt for user ID and password */
         inputUserCredentials(&userId, &passWd);
         attempts++;
      }

   } while ((ctgRc == ECI_ERR_SECURITY_ERROR) && (attempts < MAX_ATTEMPTS));

   if (ctgRc != CTG_OK) {
      printf("\nCTG_ECI_Execute_Channel call failed\n");

      displayRc(ctgRc);
      if (ctgRc == ECI_ERR_TRANSACTION_ABEND) {
         printf("Abend code : %.4s\n", eciParms.eci_abend_code);
      }
      rc = RC_FAIL;
   }

   return rc;
}


/*
   Function: closeGatewayConnection

   Purpose:  Closes the connection to the CICS TG.
*/
int closeGatewayConnection(CTG_ConnToken_t * gatewayTokenPtr) {

   int ctgRc = CTG_OK;  /* ECIv2 return code    */
   int rc = RC_OK;      /* Function return code */

   /* Close connection to CICS TG */
   ctgRc = CTG_closeGatewayConnection(gatewayTokenPtr);

   if (ctgRc != CTG_OK) {
      /* If the CTG_closeGatewayConnection function fails, either             */
      /* communication with the CICS TG has been lost, or the gateway token   */
      /* has been corrupted. If communication with the CICS TG has been lost, */
      /* then the program can probably continue safely. If the Gateway token  */
      /* has been corrupted then this indicates a more serious error.         */

      printf("Failed to close connection to CICS Transaction Gateway\n");
      displayRc(ctgRc);
      rc = RC_FAIL;
   } else {
      printf("Closed connection to CICS Transaction Gateway\n\n");
   }

   return rc;
}


/*
   Function: selectServer

   Purpose:  Queries the CICS TG for a list of available servers, and prompts
             the user to select a server. The name of the chosen server is
             copied into variable serverName.
*/
int selectServer(CTG_ConnToken_t gatewayToken, char * serverName) {

   CTG_listSystem_t servers[MAX_SERVERS];    /* CICS servers returned         */
   unsigned short serverCount = MAX_SERVERS; /* Number of CICS servers        */
   char inputBuffer[8];                      /* Buffer to receive user input  */
   int choice = 0;                           /* Server choice                 */
   int ctgRc = CTG_OK;                       /* ECIv2 return code             */
   int rc = RC_OK;                           /* Function return code          */
   int i = 0;                                /* Loop variable                 */
   

   /* Perform CTG_listSystems call */
   ctgRc = CTG_listSystems(gatewayToken, &serverCount, servers);

   if (ctgRc != CTG_OK) {
      printf("CTG_listSystems call failed\n");
      displayRc(ctgRc);
      rc = RC_FAIL;
   }

   /* Get server choice from user */
   while ((choice == 0) && (rc == RC_OK)) {

      /* Print out server list. Server names and descriptions */
      /* are null terminated.                                 */
      printf("Servers returned from CICS TG:\n");
      for (i = 0; i < serverCount; i++) {
         printf("   %d. %s - %s\n", i + 1, servers[i].SystemName,
                servers[i].Description);
      }

      /* Get user input */
      printf("\nChoose server to connect to, or q to quit:\n");
      fgets(inputBuffer, sizeof(inputBuffer), stdin);

      /* Check for quit */
      if ((inputBuffer[0] == 'q') || (inputBuffer[0] == 'Q')) {
         rc = RC_QUIT;

      } else {
         /* Validate user input */
         choice = atol(inputBuffer);
         
         if ((choice < 1) || (choice > serverCount)) {
            printf("Out of range\n");
            choice = 0;
         }
      }
   }

   if (rc == RC_OK) {
      /* Copy server name into serverName variable */
      memcpy(serverName, servers[choice - 1].SystemName,
             CTG_LIST_SYSTEM_LENGTH);
   }

   return rc;
}



/*
   Function: createChannel

   Purpose:  Creates a channel with one CHAR container to pass to the EC03
             program.
*/
int createChannel(ECI_ChannelToken_t * channelTokPtr) {

   ECI_ChannelToken_t token;
   char inputBuffer[128] = {0};  /* Buffer to receive user input */
   int ctgRc = ECI_NO_ERROR;     /* ECIv2 return code            */
   int rc = RC_OK;               /* Function return code         */

   ctgRc = ECI_createChannel(CHANNEL_NAME, &token);
   if (ctgRc == ECI_NO_ERROR) {

      /* Prompt the user to enter text to send to the CICS program */
      printf("Enter text to send to the CICS program (max %d):\n",MAX_DATA);
      fgets(inputBuffer, sizeof(inputBuffer), stdin);

      if (strlen(inputBuffer) > MAX_DATA){
    	  inputBuffer[MAX_DATA] = '\0';
    	  printf("Truncating text to use length %d\n",MAX_DATA);
      } else {
    	  /* Remove trailing newline */
          inputBuffer[strlen(inputBuffer) - 1] = '\0';
      }

      /* Create a new CHAR container with the user text, in the    */
      /* default (current) encoding.                               */
      ctgRc = ECI_createContainer(token, CONTAINER_NAME, ECI_CHAR, 0,
                                  inputBuffer, strlen(inputBuffer));

      if (ctgRc != ECI_NO_ERROR) {
         /* Container creation failed, delete the channel */
         printf("ECI_createContainer call failed\n");
         displayRc(ctgRc);
         deleteChannel(&token);
         rc = RC_FAIL;
      }

   } else {
      printf("ECI_createChannel call failed\n");
      displayRc(ctgRc);
      rc = RC_FAIL;
   }

   if (rc == RC_OK) {
      *channelTokPtr = token;
   } else {
      *channelTokPtr = ECI_NULL_CHANNELTOKEN;
   }

   return rc;
}


/*
   Function: displayChannelContents

   Purpose:  Prints a list of the containers in the channel and the contents
             of each container.
*/
int displayChannelContents(ECI_ChannelToken_t channelToken) {

   ECI_CONTAINER_INFO contInfo = {0}; /* Container information      */
   char   dataBuff[MAX_DATA +1] = {0};/* Buffer for container data  */
   size_t offset = 0;                 /* Offset into container data */
   size_t bytesRead = 0;              /* Bytes read from container  */
   size_t index = 0;                  /* Loop index                 */
   int    ctgRc = ECI_NO_ERROR;       /* ECIv2 return code          */
   int    rc = RC_OK;                 /* Function return code       */

   printf("Program EC03 returned containers:\n");

   /* Get first container from channel */
   ctgRc = ECI_getFirstContainer(channelToken, &contInfo);

   while ((ctgRc == ECI_NO_ERROR) && (rc == RC_OK)) {

      /* Display container type and name */
      if (contInfo.type == ECI_BIT) {
         printf("    [BIT] %16s = ", contInfo.name);
      } else {
         printf("   [CHAR] %16s = ", contInfo.name);
      }

      /* Display container data */
      offset = 0;
      while ((offset < contInfo.dataLength) && (ctgRc == ECI_NO_ERROR)) {

         /* Read block of data into buffer */
         ctgRc = ECI_getContainerData(channelToken, contInfo.name, dataBuff,
                                      sizeof(dataBuff), offset, &bytesRead);

         if (ctgRc == ECI_NO_ERROR) {
            /* Display data */
            if (contInfo.type == ECI_BIT) {
               for (index = 0; index < bytesRead; index++) {
                  printf("%02x", dataBuff[index]);
               }
            } else {
               printf("%.*s", (unsigned int)bytesRead, dataBuff);
            }

            offset += bytesRead;
         }
      }
      printf("\n");

      if (ctgRc != ECI_NO_ERROR) {
         printf("ECI_getContainerData call failed\n");
         displayRc(ctgRc);
         rc = RC_FAIL;
      } else {
         /* Get next container */
         ctgRc = ECI_getNextContainer(channelToken, &contInfo);
      }
   }

   if ((ctgRc != ECI_ERR_ENDOFCONTAINERS) && (rc == RC_OK)) {
      /* An error occurred */
      printf("ECI_getFirstContainer or ECI_getNextContainer call failed\n");
      displayRc(ctgRc);
      rc = RC_FAIL;
   }

   return rc;
}


/*
   Function: deleteChannel

   Purpose:  Deletes the channel
*/
int deleteChannel(ECI_ChannelToken_t * channelTokPtr) {

   int ctgRc = ECI_NO_ERROR;  /* ECIv2 return code    */
   int rc = RC_OK;            /* Function return code */

   ctgRc = ECI_deleteChannel(channelTokPtr);

   if (ctgRc != ECI_NO_ERROR) {
      printf("ECI_deleteChannel call failed.\n");
      displayRc(ctgRc);
      rc = RC_FAIL;
   }

   return rc;
}


/*
   Function: inputUserCredentials

   Purpose:  Prompts the user to enter a user ID and password or password phrase.
*/
void inputUserCredentials(char ** userId, char ** password) {

   char inputBuffer[128] = {0};  /* Buffer to receive user input */

   /* Prompt for user ID */
   printf("Enter your CICS user ID:\n");
   fgets(inputBuffer, sizeof(inputBuffer), stdin);
   inputBuffer[strlen(inputBuffer) - 1] = '\0';
   *userId = strdup(inputBuffer);

   /* Prompt for password or password phrase */
   memset(inputBuffer, 0, sizeof(inputBuffer));
   printf("Enter your CICS password or password phrase:\n");
   fgets(inputBuffer, sizeof(inputBuffer), stdin);
   inputBuffer[strlen(inputBuffer) - 1] = '\0';
   *password = strdup(inputBuffer);
}



/*
   Function: displayRc

   Purpose:  Display the return code from an API call.
*/
void displayRc(int ctgRc) {

   char rcString[CTG_MAX_RCSTRING + 1] = {0};  /* Return code string */

   /* Get the return code string */
   CTG_getRcString(ctgRc, rcString);

   /* Display return code */
   printf("Return code: %s (%d)\n", rcString, ctgRc);
}
