/*
   File Name(s)  : ctgesib1.c

   Product       : CICS Transaction Gateway

   Description   : This sample shows the basic usage of the remote support
                   in the CICS Transaction Gateway ECI API version 2. The
                   program queries the CICS TG for a list of available
                   servers, prompts for a user ID and password and then
                   verifies them with the CICS server.

   Prerequisites:  Requires a CICS server with security enabled and a protocol
                   that supports password expiry management (SNA or IPIC)

 Licensed Materials - Property of IBM  
  
 5724-I81,5725-B65,5655-Y20 
  
 (C) Copyright IBM Corp. 2011, 2014 All Rights Reserved.  
  
 US Government Users Restricted Rights - Use, duplication or  
 disclosure restricted by GSA ADP Schedule Contract with  
 IBM Corp.  

    Status: Version 9 Release 1 

                   The following code is sample code created by IBM
                   Corporation.  This sample code is not part of any
                   standard IBM product and is provided to you solely for
                   the purpose of assisting you in the development of your
                   applications.  The code is provided 'AS IS', without
                   warranty or condition of any kind.  IBM shall not be
                   liable for any damages arising out of your use of the
                   sample code, even if IBM has been advised of the
                   possibility of such damages.
*/


/* Include files */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>

/* Remote ESI support functions */
#include <ctgclient_esi.h>

#define MAX_SERVERS     40    /* Maximum number of servers to return from CICS TG */

/* Return codes used by the program */
#define RC_OK     CTG_OK         /* Return code for no error            */
#define RC_QUIT   1              /* Return code if the user enters 'q'  */
#define RC_FAIL   2              /* Return code for a failure condition */

#ifndef NULL
#define NULL 0
#endif

#define copyright_notice copyright4cctgesib1
char version4cctgesib1[] =
"\nCICS Transaction Gateway"
" Version 9.1.0.4"
" Build Level c910-20231012-1355" ;

char copyright_notice[] =
"\n"
"Licensed Materials - Property of IBM. "
"\n"
"5724-I81,5725-B65,5655-Y20 "
"\n"
"(c) Copyright IBM Corp. 2011, 2014 All Rights Reserved. "
"\n"
"US Government Users Restricted Rights - Use, duplication or "
"disclosure restricted by GSA ADP Schedule Contract with "
"IBM Corp."
"\n";


/* Function prototypes */
int openGatewayConnection(CTG_ConnToken_t* gatewayTokenOut, char* hostname, int port, int connTimeout);
int closeGatewayConnection(CTG_ConnToken_t* gatewayTokenPtr);
int selectServer(CTG_ConnToken_t gatewayToken, char* serverName);
void inputUserCredentials(char** userId, char** password);
int verifyUser(CTG_ConnToken_t gwytok, char *userId, char* password, char* server);
void displayRc(int rc);

/*
   Function: main

   Purpose:  Main function for the program. It performs the following steps:
               1. Parses command line arguments
               2. Opens a connection to the CICS TG
               3. Lists the servers defined on the CICS TG and prompts the user to select a server
               4. Prompts the user to enter a user ID and password
               5. Calls CICS to verify the user ID and password
               6. Closes the CICS TG connection
*/
int main(int argc, char** argv){
   CTG_ConnToken_t gatewayToken;             /* Gateway token                       */
   char* hostname = "localhost";             /* Host name of the CICS TG            */
   int port = 2006;                          /* Port of the CICS TG TCP/IP handler  */
   int rc = 0;                               /* Return code                         */
   int connTimeout = 60;
   char serverName[ESI_SERVER_LENGTH];
   char* userId = NULL;
   char* password = NULL;

   /* Display banner and usage */
   printf("CICS Transaction Gateway Basic ESI V2 Sample\n\n");
   printf("Usage: ctgesib1 [host name] [port number]\n\n");


   /* 1. Parse commandline arguments */
   /* Check for host name parameter */
   if (argc > 1) {
      hostname = argv[1];
   }

   /* Check for port parameter */
   if (argc > 2) {
      port = atoi(argv[2]);
      if ((port < 1) || (port > 65536)) {
         printf("Invalid port specified, defaulting to 2006\n\n");
         port = 2006;
      }
   }

   /* 2. Open connection to CICS TG */
   rc = openGatewayConnection(&gatewayToken, hostname, port, connTimeout);
   if (rc == RC_OK) {

      /* 3. List servers defined on the CICS TG and prompt the user to select a server */
      rc = selectServer(gatewayToken, serverName);
      if (rc == RC_OK) {
         /* 4. Get the user ID and password */
         inputUserCredentials(&userId, &password);
         /* 5. Verify the user ID and password */
         rc = verifyUser(gatewayToken, userId, password, serverName);

         /* After the call, release the memory allocated for user ID  */
         /* and password if they were set.                            */
         if (userId != NULL) {
            free(userId);
            userId = NULL;
         }
         if (password != NULL) {
            free(password);
            password = NULL;
         }
      }

      /* 6. Close the CICS TG connection */
      if (closeGatewayConnection(&gatewayToken) != RC_OK) {
         rc = RC_FAIL;
      }
   }

   /* Display results of program */
   if (rc == RC_FAIL) {
      printf("ctgesib1 encountered errors during execution\n");
   } else {
      printf("ctgesib1 completed successfully\n");
   }

   return rc;

}

/*
   Function: openGatewayConnection

   Purpose:  Opens a connection to the CICS TG using the host name and port provided, and if
             successful returns the gateway token in gatewayTokenOut.
*/
int openGatewayConnection(CTG_ConnToken_t* gatewayTokenOut, char* hostname, int port, int connTimeout) {

   int rc = CTG_OK;    /* Return code                                                    */

   /* Open CICS TG connection */
   rc = CTG_openRemoteGatewayConnection(hostname, port, gatewayTokenOut,connTimeout);

   if (rc != CTG_OK) {
      printf("Failed to connect to CICS TG on %s using port %d\n", hostname, port);
      displayRc(rc);
      rc = RC_FAIL;
   } else {
      printf("Connected to CICS TG on %s using port %d\n\n", hostname, port);
   }

   return rc;
}

/*
   Function: closeGatewayConnection

   Purpose:  Closes the connection to the CICS TG.
*/
int closeGatewayConnection(CTG_ConnToken_t* gatewayTokenPtr) {

   int rc = CTG_OK;  /* Return code */

   /* Close connection to CICS TG */
   rc = CTG_closeGatewayConnection(gatewayTokenPtr);

   if (rc != CTG_OK) {
      /* If the CTG_closeGatewayConnection function fails, either communication with the */
      /* CICS TG has been lost, or the gateway token has been corrupted. If communication with  */
      /* the CICS TG has been lost, then the program can probably continue safely. If the       */
      /* gateway token has been corrupted then this indicates a more serious error.             */

      printf("Failed to close connection to CICS Transaction Gateway\n");
      displayRc(rc);
      rc = RC_FAIL;
   } else {
      printf("Closed connection to CICS Transaction Gateway\n\n");
   }

   return rc;
}

/*
   Function: selectServer

   Purpose:  Queries the CICS TG for a list of available servers, and prompts the user to select a
             server. The name of the chosen server is copied into variable serverName.
*/
int selectServer(CTG_ConnToken_t gatewayToken, char* serverName) {

   CTG_listSystem_t servers[MAX_SERVERS];       /* CICS servers returned from the CICS TG */
   unsigned short serverCount = MAX_SERVERS;    /* Number of CICS servers returned        */
   char inputBuffer[8];                         /* Buffer to receive user input           */
   int choice = 0;                              /* Server choice                          */
   int rc = CTG_OK;                             /* Return code                            */
   int i = 0;                                   /* Loop variable                          */


   /* Perform CTG_listSystems call */
   rc = CTG_listSystems(gatewayToken, &serverCount, servers);

   if (rc != CTG_OK) {
      printf("CTG_listSystems call failed\n");
      displayRc(rc);
      rc = RC_FAIL;
   }

   /* Get server choice from user */
   while ((choice == 0) && (rc == RC_OK)) {

      /* Print out server list. Server names and descriptions */
      /* are null terminated.                                 */
      printf("Servers returned from CICS TG:\n");
      for (i = 0; i < serverCount; i++) {
         printf("   %d. %s - %s\n", i + 1, servers[i].SystemName,
                servers[i].Description);
      }

      /* Get user input */
      printf("\nChoose server to connect to, or q to quit:\n");
      fgets(inputBuffer, sizeof(inputBuffer), stdin);

      /* Check for quit */
      if ((inputBuffer[0] == 'q') || (inputBuffer[0] == 'Q')) {
         rc = RC_QUIT;

      } else {
         /* Validate user input */
         choice = atol(inputBuffer);

         if ((choice < 1) || (choice > serverCount)) {
            printf("Out of range\n");
            choice = 0;
         }
      }
   }

   if (rc == RC_OK) {
      /* Copy server name into serverName variable */
      memcpy(serverName, servers[choice - 1].SystemName, CTG_LIST_SYSTEM_LENGTH);
   }

   return rc;
}

/*
   Function: inputUserCredentials

   Purpose:  Prompts the user to enter a user ID and password or password phrase.
*/
void inputUserCredentials(char** userId, char** password) {

   char inputBuffer[128];  /* Buffer to receive user input */

   /* Prompt for user ID */
   memset(inputBuffer, 0, sizeof(inputBuffer));
   printf("Enter your CICS user ID:\n");
   fgets(inputBuffer, sizeof(inputBuffer), stdin);
   inputBuffer[strlen(inputBuffer) - 1] = '\0';
   *userId = strdup(inputBuffer);

   /* Prompt for password or password phrase */
   memset(inputBuffer, 0, sizeof(inputBuffer));
   printf("Enter your CICS password or password phrase:\n");
   fgets(inputBuffer, sizeof(inputBuffer), stdin);
   inputBuffer[strlen(inputBuffer) - 1] = '\0';
   *password = strdup(inputBuffer);
}

/*
   Function: displayRc

   Purpose:  Display the return code from an API call.
*/
void displayRc(int rc) {

   char rcString[CTG_MAX_RCSTRING + 1];  /* Return code string */

   /* Get the return code string */
   CTG_getRcString(rc, rcString);

   /* Display return code */
   printf("Return code: %s (%d)\n", rcString, rc);

   printf("\n");
}

/*
   Function: verifyUser

   Purpose: Attempts to verify the user ID and password on the given CICS server
*/
int verifyUser(CTG_ConnToken_t gwyTok, char* userId, char* password, char* serverName){

   int cicsRc = 0;
   int rc = RC_OK;
   ESI_DETAILS esiDetails;
   struct tm dateTime = {0};
   char dateTimeStr[18] = "";

   cicsRc = CTG_ESI_verifyPassword(gwyTok, userId, password, serverName, &esiDetails);

   if(cicsRc == ESI_NO_ERROR){
      printf("Password verified\n");
      strftime(dateTimeStr, sizeof(dateTimeStr), "%d/%m/%y %H:%M:%S", CTG_ESI_convertTime(&esiDetails.lastVerifiedTime, &dateTime));
      printf("  Account last verified on %s\n",dateTimeStr);
      if(esiDetails.expiryTime.year == 0){
         printf("  Account is non-expiring\n");
      } else {
         strftime(dateTimeStr, sizeof(dateTimeStr), "%d/%m/%y %H:%M:%S", CTG_ESI_convertTime(&esiDetails.expiryTime, &dateTime));
         printf("  Account expires on %s\n",dateTimeStr);
      }
   } else {
      printf("Failed to verify user ID and password\n");
      displayRc(cicsRc);
      rc = RC_FAIL;
   }
   return rc;
}
