/* File Name     : ecia1.cpp

   Product       : CICS Transaction Gateway

   Description   : This sample shows an advanced form of using the CICS TG
                   Client API. It queries the Client daemon for a list of
                   available servers and then a thread per server. Each thread
                   runs a user supplied number of ECI calls on that server
                   asynchronously, and works out the average response time per
                   server

   Prerequisites:  This samples requires the CICS Program EC02 to be
                   installed on your CICS server for operation.

 Licensed Materials - Property of IBM  
  
 5724-I81,5725-B65 
  
 (C) Copyright IBM Corp. 1999, 2014 All Rights Reserved.  
  
 US Government Users Restricted Rights - Use, duplication or  
 disclosure restricted by GSA ADP Schedule Contract with  
 IBM Corp.  
    
    Status: Version 9 Release 1 

                   The following code is sample code created by IBM
                   Corporation.  This sample code is not part of any
                   standard IBM product and is provided to you solely for
                   the purpose of assisting you in the development of your
                   applications.  The code is provided 'AS IS', without
                   warranty or condition of any kind.  IBM shall not be
                   liable for any damages arising out of your use of the
                   sample code, even if IBM has been advised of the
                   possibility of such damages. */


// Include files

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#if defined(CICS_W32) || defined(CICS_LNX)
#include <iostream>
#include <iomanip>
using namespace std;
#else
#include <iostream.h>
#endif

#include <cicseci.hpp>
#include "cclcalls.h"


// Defines and macros

#define ECI_SYNC_SIZE 18
#define MAX_SYSTEMS   20

#define CCLLOG(A) \
  cout << QueryMyThreadId() << "(" << Idx << "," << Thread << ") => " << A << endl;

#define CCLLOG2(A,B,C,D) printf("%d (%d,%d) => ", QueryMyThreadId(), Idx, Thread); \
                         printf(A,B,C,D);                                          \
                         printf("\n");

#if defined(CICS_W32)
#define DEBUG_OPTION	"/DEBUG"
#else
#define DEBUG_OPTION	"-DEBUG"
#endif

/*  Global Variables - If accessed by multiple threads, must be protected by
                       a mutex semaphore */

CclECI* peci;
unsigned short serverCount = MAX_SYSTEMS;//Actual number of servers in ini file
float    *average;                       //Average time per call so far
CclEvent  StartTest,                     //Event flagged to start the test
          FinishTest,                    //Event flagged by last server thread
          StartedThread;                 //Event flagged as a thread starts up
CclMutex  MainMutex;                     //MUTEX to protect access to...
int       CompletedThreads = 0 ;         //No of threads completed processing


// Global argument variables with initial values

int       ThreadsPerServer = 5;          // Threads per server (Default: 1)
int       CallsPerThread   = 10;         // No of calls per thread (Dflt: 10)
CclBool   Debug            = CCLFALSE;   // Display Debug information

#define copyright_notice copyright4cppecia1
char version4cppecia1[] =
"\nCICS Transaction Gateway"
" Version 9.1.0.4"
" Build Level c910-20231012-1355" ;

char copyright_notice[] =
"\n"
"Licensed Materials - Property of IBM. "
"\n"
"5724-I81,5725-B65 "
"\n"
"(c) Copyright IBM Corp. 1999, 2014 All Rights Reserved. "
"\n"
"US Government Users Restricted Rights - Use, duplication or "
"disclosure restricted by GSA ADP Schedule Contract with "
"IBM Corp."
"\n";

// Prototypes

void Response(CclException &ex, CclThreadId Tid);
void TerminateThread(int Idx, int Thread);
char *StrToupper(char *str);

#ifdef CICS_W32
extern "C" void __cdecl ECILoop(void *IdxPtr);
#endif

#if defined(CICS_AIX) || defined(CICS_SOL) || defined(CICS_HPUX) || defined(CICS_LNX)
extern "C" void ECILoop(void *IdxPtr);
#endif


// Main function

int main(int argc, char *argv[]){


   // Local variables

   struct Parms {
      int i;
      int j;
   } Parm;
   float RunningTotal;
   char  response;


   // Display Banner and syntax of program

   cout << "CICS Transaction Gateway Advanced ECI V1 C++ Sample\n\n";
   cout << "SYNTAX : \n";
   cout << "  ecia1 <Threads> <Calls> [" << DEBUG_OPTION << "]\n";
   cout << "    where \n";
   cout << "  Threads    = Number of threads to run per server (Default=5)\n";
   cout << "  Calls      = Number of ECI calls to make per thread (Default=10)\n";
   cout << "  " << DEBUG_OPTION << "     = Show all messages (Default is off)\n" << endl;


   // Parsing of the args

   if (argc > 1) {
	   if(atol(argv[1]) < 1) {
		   cout << "ERROR: " << "\"" << argv[1] << "\" is not a valid choice, threads per server needs to be above 0." << endl;
		   return -1;
	   }
       else ThreadsPerServer = atol(argv[1]);
   }
   if (argc > 2) {
	  if(strcmp(argv[2], "0") == 0 || atol(argv[2]) > 0) CallsPerThread   = atol(argv[2]);
	  else {
		  cout << "ERROR: " << "\"" << argv[2] << "\" is not a valid choice, ECI calls per thread needs to be 0 or above." << endl;
		  return -1;
	  }
   }
   if (argc > 3) {
	   if(strcmp(StrToupper(argv[3]), DEBUG_OPTION) == 0) Debug = CCLTRUE;
	   else {
		  cout << "ERROR: " << "\"" << argv[3] << "\" is not a valid choice." << endl;
		  return -1;
	  }
   }


   // Create the ECI Object and get the number of servers defined.

   peci = CclECI::instance();

   try {
    serverCount = peci->serverCount();
   } catch ( CclException &ex ) {
     cout << "ECI serverCount request failed" << endl;
     Response(ex,QueryMyThreadId());
     //If we can't obtain the number of servers defined exit.
     exit(1);      
   }

   cout << "Running with: \n";
   cout << serverCount << " Servers\n";
   cout << ThreadsPerServer << " Threads per server\n";
   cout << CallsPerThread << " ECI calls per thread" << endl;


   // Create a Mutex and some Event semaphores

   if (CreateAnEvent("StartedThread", &StartedThread) != CCLTRUE) {
      cout  << "MAIN : Failed to create thread started event - Aborting."
            << endl;
      return 1;
   }
   if (CreateAMutex("MainMutex", &MainMutex) != CCLTRUE) {
      cout << "MAIN : Failed to create main mutex - Aborting." << endl;
      return 2;
   }
   if (CreateAnEvent("StartTest", &StartTest) != CCLTRUE) {
      cout << "MAIN : Failed to create StartTest event - Aborting." << endl;
      return 3;
   }
   if (CreateAnEvent("FinishTest", &FinishTest) != CCLTRUE) {
      cout << "MAIN : Failed to create FinishTest event - Aborting." << endl;
      return 4;
   }


   // Malloc storage for the averages

   average = new float[serverCount * ThreadsPerServer];

   if (average == CCLNULL) {
      cout << "MAIN : Failed to allocate storage - Aborting." << endl;
      return 5;
   }


   // Start a thread on a per-server basis

   for (Parm.i = 0; Parm.i < serverCount; Parm.i++) {
      for (Parm.j = 0; Parm.j < ThreadsPerServer; Parm.j++) {
         if (StartAThread( ECILoop, &Parm ) != CCLTRUE) {
            cout << "MAIN : Failed to create thread " << Parm.i << "/"
                 << Parm.j << " - Aborting." << endl;
            return 6;
         }

         if (Debug == CCLTRUE) cout << "MAIN : Waiting for new thread to start"
                                    << endl;

         if (WaitOnAnEvent( &StartedThread ) != CCLTRUE) {
            cout << "MAIN : Failed to wait for thread " << Parm.i << "/"
                 << Parm.j << " to start - Aborting." << endl;
            return 7;
         }

         // Reset the event for reuse
         if (ResetAnEvent( &StartedThread ) != CCLTRUE) {
            cout << "MAIN : Failed to wait for thread " << Parm.i
                 << "/" << Parm.j << " to start - Aborting." << endl;
            return 8;
         }

         if (Debug == CCLTRUE) cout << "MAIN : Waiting for new thread to start"
                                    << endl;
      }
   }
   cout << "MAIN : Threads all started - Starting test." << endl;


   // All threads have started and are waiting to go. Post the StartTest event

   if (FlagAnEvent( &StartTest ) != CCLTRUE) {
      cout << "MAIN : Failed to post the StartTest flag - Aborting." << endl;
      return 9;
   }


   // Wait for a second

   WaitInMillisecs(1000);


   // Wait for Test Complete Flag

   if (WaitOnAnEvent( &FinishTest ) != CCLTRUE) {
      cout << "MAIN : Failed to wait on the FinishTest flag - Aborting."
           << endl;
      return 10;
   }


   // Display results

   cout << "MAIN : Threads all finished - Results follow:" << endl;
   for (Parm.i = 0; Parm.i < serverCount; Parm.i++) {
      RunningTotal = 0.0;
      for (Parm.j = 0; Parm.j < ThreadsPerServer; Parm.j++) {
         if (Debug == CCLTRUE) {
            cout << "Server : " << peci->serverName(Parm.i + 1) << ", thread "
                 << Parm.j << " average time in seconds " <<
            ((float) average[(Parm.i*ThreadsPerServer)+Parm.j])/1000.0 <<
            endl;
         }

         RunningTotal = RunningTotal +
                               average[(Parm.i*ThreadsPerServer)+Parm.j];
      }
      cout << "Server : " << peci->serverName(Parm.i + 1) <<
      ", overall average time in seconds " <<
      RunningTotal / (1000.0 * (float) ThreadsPerServer) << endl;
   }

   cout <<  "Press enter to terminate" << endl;
   cin.get(response);



   // Clean up

   delete[] average;

   DeleteAnEvent( &StartTest );
   DeleteAnEvent( &FinishTest );
   DeleteAnEvent( &StartedThread );
   DeleteAMutex(  &MainMutex );

   return 0;
}


// Function started per server thread

extern "C" void ECILoop(void *IdxPtr) {


   // Variable definitions

   char UserID[9] = "sysad";                      //UserID
   char PassWd[9] = "sysad";                      //Password
   char progid[4 + 1] = "EC01";                   //Users requested transaction
   char CallBackEvt[20];

   struct Parms {
      int IdxOfServer;
      int ThreadPerServer;
   } *Parm;

   int Idx, Thread, AIdx;
   int i   = 0;

   unsigned long TimeBefore, TimeAfter;


   /* get the server index number, Thread index, and a unique index for this
      server/thread combination. */

   Parm = (struct Parms *) IdxPtr;
   Idx     = Parm->IdxOfServer;
   Thread  = Parm->ThreadPerServer;
   AIdx    = (Idx * ThreadsPerServer) + Thread;

   if (Debug == CCLTRUE) CCLLOG("Thread started");


   // Inform main thread we have read the value in 'i'

   if (FlagAnEvent( &StartedThread) != CCLTRUE) {
      CCLLOG("Failed to inform main thread via StartedThread - Aborting");
      TerminateThread(Idx, Thread);
      return;
   }


   // Wait until told to start

   if (WaitOnAnEvent( &StartTest ) != CCLTRUE) {
      CCLLOG("Failed to wait on the StartTest flag - Aborting");
      TerminateThread(Idx, Thread);
      return;
   }


   // Loop running the tests, calculating the average time

   average[AIdx] = 0.0;

   if (Debug == CCLTRUE) CCLLOG("Running the test");

   CclConn* pConn;
   CclFlow* pFlow;
   CclBuf*  Commarea;

   pConn = new CclConn(peci->serverName(Idx + 1),UserID,PassWd);
   Commarea = new CclBuf();
   pFlow = new CclFlow(Ccl::sync);

   for (i = 0; i<CallsPerThread; i++) {
      Commarea->setDataLength(ECI_SYNC_SIZE);


      // Call the chosen server with the given parameters.

      TimeBefore = CurTimeInMilliSecs();
      try {
         pConn->link(*pFlow,progid,Commarea);

         TimeAfter = CurTimeInMilliSecs();
         if (Debug == CCLTRUE) {
            CCLLOG2("Eci call %d, Commarea =%s %s", i,
                              (char*)Commarea->dataArea(),".");
         }
         average[AIdx] = ((average[AIdx] * (i)) + (TimeAfter-TimeBefore))
                                    / (i+1);
      } catch (CclException &ex) {


         /* if an exception occurs, assume it will happen on every link call there
            after so just terminate the thread */

         Response(ex,QueryMyThreadId());
         delete pFlow;
         delete Commarea;
         delete pConn;
         TerminateThread(Idx, Thread);
      }
   }

   delete pFlow;
   delete Commarea;
   delete pConn;
   TerminateThread(Idx, Thread);

}


/* Function: Response

   Purpose:  Display the immediate response code from an ECI call */

void Response(CclException &ex, CclThreadId Tid)
{
   if (Tid != 0) {
      cout << Tid << " : ECI returned: " <<  ex.exCodeText() << endl;
   } else {
      cout << "ECI returned: " << ex.exCodeText() << endl;
   }

   if (ex.exCode() != Ccl::noError) {
      if (Tid != 0) {
         cout << Tid << " : Abend code was \"" << ex.abendCode() << "\""
              << endl;
      } else {
         cout << "Abend code was \"" << ex.abendCode() << "\"" << endl;
      }
   }
}


/* Function: TerminateThread

   Purpose:  To increment the thread termination count if it all threads have
             ended, then flag the main thread. */

void TerminateThread(int Idx, int Thread)
{
   CclBool AllFinished = CCLFALSE;

   if (GetExclusiveAccess( &MainMutex ) != CCLTRUE) {
      CCLLOG("Failed to get the exclusive access - Aborting");
      EndMyThread();
      return;
   }

   CompletedThreads = CompletedThreads + 1;
   if (CompletedThreads == (serverCount * ThreadsPerServer)) {
      AllFinished = CCLTRUE;
   }

   if (ReleaseExclusiveAccess( &MainMutex ) != CCLTRUE) {
      CCLLOG("Failed to release the exclusive access - Aborting");
      EndMyThread();
      return;
   }

   if (AllFinished == CCLTRUE) {

      // Inform main thread all threads are done

      if (FlagAnEvent( &FinishTest) != CCLTRUE) {
         CCLLOG("Failed to inform main thread via FinishThread - Aborting");
         EndMyThread();
         return;
      }
   }

   EndMyThread();
}



/* Function: StrToupper

   Purpose:  Takes an array of characters and convert them to upper case */

char *StrToupper(char *str)
{
        int i = 0;

        for(i = 0; i < strlen(str); i++)
                if (str[i] >= 'a' && str[i] <= 'z') str[i] = str[i] - 32;

        return str;
}
