/* File Name(s)  : epia1.cpp

   Product       : CICS Transaction Gateway

   Description   : This sample shows an advanced form of using the CICS TG C
                   API. It queries the Client daemon for a list of available  
                   servers and then creates a number of threads per server.
                   Each thread runs a user supplied number of EPI transactions
                   on that server, and works out the average response time per
                   server.

   Prerequisites:  This sample requires the CICS Transaction EP01 to be
                   installed on your CICS server for operation.

 Licensed Materials - Property of IBM  
  
 5724-I81,5725-B65 
  
 (C) Copyright IBM Corp. 1999, 2014 All Rights Reserved.  
  
 US Government Users Restricted Rights - Use, duplication or  
 disclosure restricted by GSA ADP Schedule Contract with  
 IBM Corp.  
   
   Status: Version 9 Release 1 

                   The following code is sample code created by IBM
                   Corporation.  This sample code is not part of any standard
                   IBM product and is provided to you solely for the purpose
                   of assisting you in the development of your applications.
                   The code is provided 'AS IS', without warranty or
                   condition of any kind.  IBM shall not be liable for any
                   damages arising out of your use of the sample code, even
                   if IBM has been advised of the possibility of such damages. */



// Includes and Defines

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#if defined(CICS_W32) || defined(CICS_LNX)
#include <iostream>
#include <iomanip>
using namespace std;
#else
#include <iostream.h>
#endif

#include <cicsepi.hpp>
#include "cclcalls.h"

#define MAX_SYSTEMS   20

#define CCLLOG(A) \
  cout << QueryMyThreadId() << "(" << Idx << "," << Thread << ") => " << A << endl;

#define CCLLOG2(A,B,C,D) printf("%d (%d,%d) => ", QueryMyThreadId(), Idx, Thread);  \
                         printf(A,B,C,D);                                           \
                         printf("\n");

#if defined(CICS_W32)
#define DEBUG_OPTION	"/DEBUG"
#else
#define DEBUG_OPTION	"-DEBUG"
#endif

/*  Global Variables - If accessed by multiple threads, must be protected by
                       a mutex semaphore */

unsigned short serverCount = MAX_SYSTEMS;//Actual number of servers in ini file
float    *average;                       //Average time per call so far
CclEvent  StartTest,                     //Event flagged to start the test
FinishTest,                              //Event flagged by last server thread
StartedThread;                           //Event flagged as a server thread starts up
CclMutex  MainMutex;                     //MUTEX to protect access to...
int       CompletedThreads = 0;          //Number of threads completed processing
CclEPI*   pepi;


// Saved copies of initial args

int       ThreadsPerServer = 5;         // Threads per server (Default: 1)
int       CallsPerThread   = 10;        // Number of calls per thread (Default: 10)
CclBool   Debug            = CCLFALSE;  // Display Debug information


// Prototypes

void Response(CclException &ex, CclThreadId Tid);
void TerminateThread(int Idx, int Thread);
char *StrToupper(char *str);

#ifdef CICS_W32
extern "C" void __cdecl EPILoop(void *IdxPtr);
#endif

#if defined(CICS_AIX) || defined(CICS_SOL) || defined(CICS_HPUX) || defined(CICS_LNX)
extern "C" void EPILoop(void *IdxPtr);
#endif

#define copyright_notice copyright4cppepia1
char version4cppepia1[] =
"\nCICS Transaction Gateway"
" Version 9.1.0.4"
" Build Level c910-20231012-1355" ;

char copyright_notice[] =
"\n"
"Licensed Materials - Property of IBM. "
"\n"
"5724-I81,5725-B65 "
"\n"
"(c) Copyright IBM Corp. 1999, 2014 All Rights Reserved. "
"\n"
"US Government Users Restricted Rights - Use, duplication or "
"disclosure restricted by GSA ADP Schedule Contract with "
"IBM Corp."
"\n";


// Main function

int main(int argc, char *argv[]){
   struct Parms {
      int i;
      int j;
   } Parm;
   float RunningTotal;
   char  response;


   // Display Banner and syntax of program

   cout << "CICS Transaction Gateway Advanced EPI C++ Sample\n\n";
   cout << "SYNTAX : \n";
   cout << "  epia1 <Threads> <Calls> [" << DEBUG_OPTION << "]\n";
   cout << "    where \n";
   cout << "  Threads    = Number of threads to run per server (Default=5)\n";
   cout << "  Calls      = Number of EPI transactions to make per thread (Default=10)\n";
   cout << "  " << DEBUG_OPTION << "     = Show all messages (Default is off)\n" << endl;


   // Parsing of the args

   if (argc > 1) {
	   if(atol(argv[1]) < 1) {
		   cout << "ERROR: " << "\"" << argv[1] << "\" is not a valid choice, threads per server needs to be above 0.";
		   return -1;
	   }
       else ThreadsPerServer = atol(argv[1]);
   }
   if (argc > 2) {
	  if(strcmp(argv[2], "0") == 0 || atol(argv[2]) > 0) CallsPerThread   = atol(argv[2]);
	  else {
		  cout << "ERROR: " << "\"" << argv[2] << "\" is not a valid choice, EPI calls per thread needs to be 0 or above.";
		  return -1;
	  }
   }
   if (argc > 3) {
	   if(strcmp(StrToupper(argv[3]), DEBUG_OPTION) == 0) Debug = CCLTRUE;
	   else {
		  cout << "ERROR: " << "\"" << argv[3] << "\" is not a valid choice.";
		  return -1;
	  }
   }


   // Initialize to EPI

   try {
      pepi = new CclEPI();
   } catch (CclException &ex) {
      cout << "Exception occurred trying to create EPI object." << endl;
      Response(ex,0);
      cout << "Program cannot continue.\n" << endl;
      exit(0);
   }


   // Ask the Client daemon for a list of servers to connect to.

   try {
      serverCount = pepi->serverCount();
   } catch ( CclException &ex ) {
     cout << "EPI serverCount request failed." << endl;
     Response(ex,QueryMyThreadId());
     //Unable to obtain the number of servers defined, terminate process.
     exit(1);      
   }

   cout << "Running with: " << endl;
   cout << serverCount << " servers" << endl;
   cout << ThreadsPerServer << " threads per server" << endl;
   cout << CallsPerThread << " EPI transactions per thread." << endl;


   // Create a Mutex and some Event semaphores

   if (CreateAnEvent("StartedThread", &StartedThread) != CCLTRUE) {
      cout << "MAIN : Failed to create thread started event - aborting." << endl;
      return 1;
   }
   if (CreateAMutex("MainMutex", &MainMutex) != CCLTRUE) {
      cout << "MAIN : Failed to create main mutex - aborting." << endl;
      return 2;
   }
   if (CreateAnEvent("StartTest", &StartTest) != CCLTRUE) {
      cout << "MAIN : Failed to create StartTest event - aborting." << endl;
      return 3;
   }
   if (CreateAnEvent("FinishTest", &FinishTest) != CCLTRUE) {
      cout << "MAIN : Failed to create FinishTest event - aborting.\n" << endl;
      return 4;
   }


   // allocate storage for the averages

   average = new float[serverCount * ThreadsPerServer];

   if (average == CCLNULL) {
      cout << "MAIN : Failed to allocate storage - aborting." << endl;
      return 5;
   }


   // Start a thread on a per-server basis

   for (Parm.i = 0; Parm.i < serverCount; Parm.i++) {
      for (Parm.j = 0; Parm.j < ThreadsPerServer; Parm.j++) {
         if (StartAThread( EPILoop, &Parm ) != CCLTRUE) {     // Start the thread
            cout << "MAIN : Failed to create thread " << Parm.i << "/" << Parm.j
            << " - aborting." << endl;
            return 6;
         }
         if (Debug == CCLTRUE) {
            cout << "MAIN : Waiting for new thread to start." << endl;
         }
         if (WaitOnAnEvent( &StartedThread ) != CCLTRUE) { // Wait for it to start
            cout << "MAIN : Failed to wait for thread " << Parm.i << "/" << Parm.j
            << " - aborting." << endl;
            return 7;
         }
         if (ResetAnEvent( &StartedThread ) != CCLTRUE) {  // Reset the event for reuse
            cout << "MAIN : Failed to wait for thread " << Parm.i << "/" << Parm.j
            << " to start - aborting." << endl;
            return 8;
         }
         if (Debug == CCLTRUE) cout << "MAIN : Waiting for new thread to start."
            << endl;
      }
   }
   cout << "MAIN : Threads all started - starting test" << endl;


   // All threads have started and are waiting to go. Post the StartTest event

   if (FlagAnEvent( &StartTest ) != CCLTRUE) {
      cout << "MAIN : Failed to post the StartTest flag - aborting." << endl;
      return 9;
   }


   // Wait for a second

   WaitInMillisecs(1000);


   // Wait for Test Complete Flag

   if (WaitOnAnEvent( &FinishTest ) != CCLTRUE) {
      cout << "MAIN : Failed to wait on the FinishTest flag - aborting." << endl;
      return 10;
   }


   /* Display results. */

   cout << "MAIN : Threads all finished - Results follow:" << endl;
   for (Parm.i = 0; Parm.i < serverCount; Parm.i++) {
      RunningTotal = 0.0;
      for (Parm.j = 0; Parm.j < ThreadsPerServer; Parm.j++) {
         if (Debug == CCLTRUE) {
            cout << "Server : " << pepi->serverName(Parm.i + 1) <<
            ", thread " << Parm.j <<
            " average time in seconds " <<
            ((float) average[(Parm.i * ThreadsPerServer) + Parm.j]) / 1000.0 <<
            endl;
         }
         RunningTotal = RunningTotal + average[(Parm.i * ThreadsPerServer) + Parm.j];
      }
      cout << "Server : " << pepi->serverName(Parm.i + 1) <<
      ", overall average time in seconds " <<
      RunningTotal / (1000.0 * (float) ThreadsPerServer) <<
      endl;
   }

   cout << "Press enter to terminate." << endl;
   cin.get(response);


   // Clean up

   delete[] average;
   DeleteAnEvent( &StartTest );
   DeleteAnEvent( &FinishTest );
   DeleteAnEvent( &StartedThread );
   DeleteAMutex(  &MainMutex );

   delete pepi;

   return 0;
}


// Function started per server thread

extern "C" void EPILoop(void *IdxPtr) {

   CclSession *psess;
   CclTerminal *pterm;
   CclScreen *pscr;
   CclField *pfld;

   const char* UserID = "SYSAD";    //User ID
   const char* PassWd = "SYSAD";    //Password

   struct Parms {
      int IdxOfServer;
      int ThreadPerServer;
   } *Parm;

   int Idx, Thread, AIdx;
   int i   = 0;

   unsigned long TimeBefore, TimeAfter;

   Parm = (struct Parms *) IdxPtr;
   Idx     = Parm->IdxOfServer;
   Thread  = Parm->ThreadPerServer;
   AIdx    = (Idx * ThreadsPerServer) + Thread;

   if (Debug == CCLTRUE) CCLLOG("Thread started.");


   // Inform main thread we have read the value in 'i'

   if (FlagAnEvent( &StartedThread) != CCLTRUE) {
      CCLLOG("Failed to inform main thread via StartedThread - aborting.");
      TerminateThread(Idx, Thread);
      return;
   }


   // Wait until told to start

   if (WaitOnAnEvent( &StartTest ) != CCLTRUE) {
      CCLLOG("Failed to wait on the StartTest flag - aborting.");
      TerminateThread(Idx, Thread);
      return;
   }


   // Loop running the tests, calculating the average time

   average[AIdx] = 0.0;

   if (Debug == CCLTRUE) CCLLOG("Running the test.");

   psess = new CclSession(Ccl::sync);


   // Add a terminal

   if (Debug == CCLTRUE) CCLLOG("Adding a terminal");
   pterm = new CclTerminal(pepi->serverName(Idx + 1),"","",
                           CclTerminal::signonIncapable,UserID,PassWd,0,0);

   try {
      pterm->install(psess,0);
   }
   catch (CclException &ex) {
      CCLLOG("AddTerminal failed - aborting.");
      Response(ex, QueryMyThreadId());
      TerminateThread(Idx, Thread);
      delete pterm;
      delete psess;
      return;
   }

   pscr = pterm->screen();

   if (Debug == CCLTRUE) {

      // Output some information about the terminal.

      CCLLOG2("EPI Terminal connected to server %s, termid is %s, netname is %s",
              pepi->serverName(Idx + 1),pterm->termID(),pterm->netName());
      CCLLOG2("   Screen Info%s, dimensions %d x %d\n",":",
              pscr->depth(), pscr->width());
   }


   // Run the transaction

   for (i = 0; i < CallsPerThread; i++) {

      TimeBefore = CurTimeInMilliSecs();


      // Start transaction

      if (Debug == CCLTRUE) CCLLOG("Starting EP02.");

      try {
         pterm->send(psess,"EP02");
      }
      catch (CclException &ex) {
         CCLLOG("StartTran failed - aborting.");
         Response(ex, QueryMyThreadId());
         pterm->disconnect();
         delete pterm;
         delete psess;
         TerminateThread(Idx, Thread);
         return;
      }

      TimeAfter = CurTimeInMilliSecs();
      average[AIdx] = ((average[AIdx] * (i)) + (TimeAfter-TimeBefore)) / (i+1);
   }


   // Delete the terminal

   if (Debug == CCLTRUE) CCLLOG("Deleting terminal");
   try {
      pterm->disconnect();
   } catch (CclException &ex) {
      Response(ex, QueryMyThreadId());
   }

   delete pterm;
   delete psess;
   TerminateThread(Idx, Thread);
}


/* Function: Response

   Purpose:  Display the immediate response code from an EPI call */

void Response(CclException &ex, CclThreadId Tid)
{
   if (Tid != 0) {
      cout << Tid << " : EPI returned: " << ex.exCodeText() <<endl;
   } else {
      cout << "EPI returned: " << ex.exCodeText() << endl;
   }
}

/* Function: TerminateThread

   Purpose:  To increment the thread termination count if it all threads have
             ended, then flag the main thread. */

void TerminateThread(int Idx, int Thread)
{
   CclBool AllFinished = CCLFALSE;

   if (GetExclusiveAccess( &MainMutex ) != CCLTRUE) {
      CCLLOG("Failed to get the exclusive access - aborting.");
      EndMyThread();
      return;
   }

   CompletedThreads = CompletedThreads + 1;
   if (CompletedThreads == (serverCount * ThreadsPerServer)) {
      AllFinished = CCLTRUE;
   }

   if (ReleaseExclusiveAccess( &MainMutex ) != CCLTRUE) {
      CCLLOG("Failed to release the exclusive access - aborting.");
      EndMyThread();
      return;
   }

   if (AllFinished == CCLTRUE) {

      // Inform main thread all threads are done

      if (FlagAnEvent( &FinishTest) != CCLTRUE) {
         CCLLOG("Failed to inform main thread via FinishThread - aborting.");
         if (ReleaseExclusiveAccess( &MainMutex ) != CCLTRUE) {
            CCLLOG("Failed to release the exclusive access - aborting.");
         }
         EndMyThread();
         return;
      }
   }

   EndMyThread();
}

/* Function: StrToupper

   Purpose:  Takes an array of characters and convert them to upper case */

char *StrToupper(char *str)
{
        int i = 0;

        for(i = 0; i < strlen(str); i++)
                if (str[i] >= 'a' && str[i] <= 'z') str[i] = str[i] - 32;

        return str;
}
