/*
   MODULE NAME       CICSEPIX.H

   DESCRIPTIVE NAME  CICS EPI Exit Definitions

 Licensed Materials - Property of IBM  
  
 "Restricted Materials of IBM"  
  
 5724-I81,5725-B65 
  
 (C) Copyright IBM Corp. 1994, 2014  
  
 US Government Users Restricted Rights - Use, duplication or  
 disclosure restricted by GSA ADP Schedule Contract with  
 IBM Corp.  
   
   Status: Version 9 Release 1 

   NOTES :-

   This header file contains definitions and structures for the EPI
   exits.
*/

#ifndef _CICSEPIX_H
#define _CICSEPIX_H

#include <cicstype.h>
#include <cics_epi.h>

#ifdef __cplusplus
  extern "C" {
#endif

#if defined(CICS_AIX)
#pragma options align=packed
#elif defined(CICS_HPUX)
#pragma pack 1
#else
#pragma pack(1)
#endif

/*
   Special return codes from exits.
   All other non-zero return codes indicate an exit has failed
   but will not cause special handling in the EPI code.
*/
#define CICS_EXIT_OK                     0
#define CICS_EXIT_NO_EXITS               1 /* do not call any other exits */
#define CICS_EXIT_CANT_INIT_EXITS        2 /* cannot init exit environment */
#define CICS_EXIT_DONT_ADD_TERMINAL      3 /* abort current AddTerminal */
                                           /* or SystemId addterminal retry */

/*
   The following values correspond to error cases identified in the
   exits architecture.  They have no special meaning to the EPI
*/
#define CICS_EXIT_BAD_ANCHOR             10 /* Bad anchor value */
#define CICS_EXIT_BAD_STORAGE            11 /* Can't tidy up storage */
#define CICS_EXIT_BAD_PARM               12 /* Bad parameter */

#define CICS_EXIT_USER_BASE             100 /* exit specific codes start here */

/*
   Datatypes
*/
typedef void CICSPTR *CICS_EpiAnchor_t;

/*
   InitializeExit
   Called after EPI internal initialisation is complete
*/
typedef cics_sshort_t CICSEXIT CICS_EpiInitializeExitFn
                        ( cics_ulong_t              Version
                        , CICS_EpiAnchor_t CICSPTR *Anchor
                        );
cics_sshort_t CICSEXIT CICS_EpiInitializeExit
                        ( cics_ulong_t              Version
                        , CICS_EpiAnchor_t CICSPTR *Anchor
                        );

/*
   TerminateExit
   Called before termination
*/
typedef cics_sshort_t CICSEXIT CICS_EpiTerminateExitFn
                        ( CICS_EpiAnchor_t Anchor
                        );
cics_sshort_t CICSEXIT CICS_EpiTerminateExit
                        ( CICS_EpiAnchor_t Anchor
                        );

/*
   AddTerminalExit
   NOTE: May not be driven if AddTerminal fails with ERR_MAX_TERMINALS
*/
typedef cics_sshort_t CICSEXIT CICS_EpiAddTerminalExitFn
                        ( CICS_EpiAnchor_t     Anchor
                        , cics_char_t CICSPTR *NameSpace
                        , cics_char_t          System[CICS_EPI_SYSTEM_MAX+1]
                        , cics_char_t          NetName[CICS_EPI_NETNAME_MAX+1]
                        , cics_char_t          DevType[CICS_EPI_DEVTYPE_MAX+1]
                        );
cics_sshort_t CICSEXIT CICS_EpiAddTerminalExit
                        ( CICS_EpiAnchor_t     Anchor
                        , cics_char_t CICSPTR *NameSpace
                        , cics_char_t          System[CICS_EPI_SYSTEM_MAX+1]
                        , cics_char_t          NetName[CICS_EPI_NETNAME_MAX+1]
                        , cics_char_t          DevType[CICS_EPI_DEVTYPE_MAX+1]
                        );

/*
   StartTranExit
   This prototype is provided for compatibility with older applications.
   All new applications that use the exits should use StartTranExtendedExit
   instead of StartTranExit.
*/
typedef cics_sshort_t CICSEXIT CICS_EpiStartTranExitFn
                        ( CICS_EpiAnchor_t      Anchor
                        , cics_char_t           TransId[CICS_EPI_TRANSID_MAX+1]
                        , cics_ubyte_t CICSPTR *Data
                        , cics_ushort_t         Size
                        );
cics_sshort_t CICSEXIT CICS_EpiStartTranExit
                        ( CICS_EpiAnchor_t      Anchor
                        , cics_char_t           TransId[CICS_EPI_TRANSID_MAX+1]
                        , cics_ubyte_t CICSPTR *Data
                        , cics_ushort_t         Size
                        );

/*
   StartTranExtendedExit
   This function replaces StartTranExit.
*/
typedef cics_sshort_t CICSEXIT CICS_EpiStartTranExtendedExitFn
                        ( CICS_EpiAnchor_t      Anchor
                        , cics_ushort_t         TermIndex
                        , cics_char_t           TransId[CICS_EPI_TRANSID_MAX+1]
                        , cics_ubyte_t CICSPTR *Data
                        , cics_ushort_t         Size
                        );
cics_sshort_t CICSEXIT CICS_EpiStartTranExtendedExit
                        ( CICS_EpiAnchor_t      Anchor
                        , cics_ushort_t         TermIndex
                        , cics_char_t           TransId[CICS_EPI_TRANSID_MAX+1]
                        , cics_ubyte_t CICSPTR *Data
                        , cics_ushort_t         Size
                        );

/*
   ReplyExit
*/
typedef cics_sshort_t CICSEXIT CICS_EpiReplyExitFn
                        ( CICS_EpiAnchor_t      Anchor
                        , cics_ushort_t         TermIndex
                        , cics_ubyte_t CICSPTR *Data
                        , cics_ushort_t         Size
                        );
cics_sshort_t CICSEXIT CICS_EpiReplyExit
                        ( CICS_EpiAnchor_t      Anchor
                        , cics_ushort_t         TermIndex
                        , cics_ubyte_t CICSPTR *Data
                        , cics_ushort_t         Size
                        );

/*
   DelTerminalExit
*/
typedef cics_sshort_t CICSEXIT CICS_EpiDelTerminalExitFn
                        ( CICS_EpiAnchor_t Anchor
                        , cics_ushort_t    TermIndex
                        );
cics_sshort_t CICSEXIT CICS_EpiDelTerminalExit
                        ( CICS_EpiAnchor_t Anchor
                        , cics_ushort_t    TermIndex
                        );
/*
   GetEventExit
*/
typedef cics_sshort_t CICSEXIT CICS_EpiGetEventExitFn
                        ( CICS_EpiAnchor_t             Anchor
                        , cics_ushort_t                TermIndex
                        , CICS_EpiWait_t               Wait
                        , CICS_EpiEventData_t CICSPTR *Event
                        );
cics_sshort_t CICSEXIT CICS_EpiGetEventExit
                        ( CICS_EpiAnchor_t             Anchor
                        , cics_ushort_t                TermIndex
                        , CICS_EpiWait_t               Wait
                        , CICS_EpiEventData_t CICSPTR *Event
                        );

/*
   TranFailedExit
   This will only be driven when GetEvent returns with a genuine event
   (ie not after parameter validation errors or an exception)
*/
typedef cics_sshort_t CICSEXIT CICS_EpiTranFailedExitFn
                        ( CICS_EpiAnchor_t             Anchor
                        , cics_ushort_t                TermIndex
                        , CICS_EpiWait_t               Wait
                        , CICS_EpiEventData_t CICSPTR *Event
                        );
cics_sshort_t CICSEXIT CICS_EpiTranFailedExit
                        ( CICS_EpiAnchor_t             Anchor
                        , cics_ushort_t                TermIndex
                        , CICS_EpiWait_t               Wait
                        , CICS_EpiEventData_t CICSPTR *Event
                        );

/*
   SystemIdExit
   NOTE: Not driven if AddTerminal fails with ERR_MAX_TERMINALS
*/
typedef cics_sshort_t CICSEXIT CICS_EpiSystemIdExitFn
                    ( CICS_EpiAnchor_t     Anchor
                    , cics_char_t CICSPTR *NameSpace
                    , cics_char_t          System[CICS_EPI_SYSTEM_MAX+1]
                    , cics_char_t          NetName[CICS_EPI_NETNAME_MAX+1]
                    , cics_char_t          DevType[CICS_EPI_DEVTYPE_MAX+1]
                    , cics_char_t          FailedSystem [CICS_EPI_SYSTEM_MAX+1]
                    , cics_sshort_t        Reason
                    , cics_ulong_t         SubReason
                    , cics_char_t          UserId[64]
                    , cics_char_t          PassWord[64]
                    );
cics_sshort_t CICSEXIT CICS_EpiSystemIdExit
                    ( CICS_EpiAnchor_t     Anchor
                    , cics_char_t CICSPTR *NameSpace
                    , cics_char_t          System[CICS_EPI_SYSTEM_MAX+1]
                    , cics_char_t          NetName[CICS_EPI_NETNAME_MAX+1]
                    , cics_char_t          DevType[CICS_EPI_DEVTYPE_MAX+1]
                    , cics_char_t          FailedSystem [CICS_EPI_SYSTEM_MAX+1]
                    , cics_sshort_t        Reason
                    , cics_ulong_t         SubReason
                    , cics_char_t          UserId[64]
                    , cics_char_t          PassWord[64]
                    );

/*
   TermIdExit
   This will be driven at the end of a successful CICS_EpiAddTerminal
   This prototype is provided for compatibility with older applications.
   All new applications that use the exits should use TermIdInfoExit instead
   of TermIdExit.
*/
typedef cics_sshort_t CICSEXIT CICS_EpiTermIdExitFn
                        ( CICS_EpiAnchor_t     Anchor
                        , cics_ushort_t        TermIndex
                        , cics_char_t CICSPTR *System
                        );

cics_sshort_t CICSEXIT CICS_EpiTermIdExit
                        ( CICS_EpiAnchor_t     Anchor
                        , cics_ushort_t        TermIndex
                        , cics_char_t CICSPTR *System
                        );

/*
   TermIdInfoExit
   This will be driven at the end of a successful CICS_EpiAddTerminal
   This function replaces TermIdExit.
*/
typedef cics_sshort_t CICSEXIT CICS_EpiTermIdInfoExitFn
                        ( CICS_EpiAnchor_t            Anchor
                        , cics_ulong_t                Version
                        , cics_ushort_t              *TermIndex
                        , CICS_EpiDetails_t          *Details
                        );

cics_sshort_t CICSEXIT CICS_EpiTermIdInfoExit
                        ( CICS_EpiAnchor_t            Anchor
                        , cics_ulong_t                Version
                        , cics_ushort_t              *TermIndex
                        , CICS_EpiDetails_t          *Details
                        );

typedef struct CICS_EpiExitList
{
    CICS_EpiInitializeExitFn        CICSPTR *InitializeExit;
    CICS_EpiTerminateExitFn         CICSPTR *TerminateExit;
    CICS_EpiAddTerminalExitFn       CICSPTR *AddTerminalExit;
    CICS_EpiStartTranExitFn         CICSPTR *StartTranExit;
    CICS_EpiReplyExitFn             CICSPTR *ReplyExit;
    CICS_EpiDelTerminalExitFn       CICSPTR *DelTerminalExit;
    CICS_EpiGetEventExitFn          CICSPTR *GetEventExit;
    CICS_EpiTranFailedExitFn        CICSPTR *TranFailedExit;
    CICS_EpiSystemIdExitFn          CICSPTR *SystemIdExit;
    CICS_EpiTermIdExitFn            CICSPTR *TermIdExit;
    CICS_EpiTermIdInfoExitFn        CICSPTR *TermIdInfoExit;
    CICS_EpiStartTranExtendedExitFn CICSPTR *StartTranExtendedExit;
}   CICS_EpiExitList_t;

#if defined(CICS_AIX)
#pragma options align=reset
#elif defined(CICS_HPUX) && defined(__cplusplus)
#pragma pack
#elif defined(CICS_HPUX) && defined(CICS_HPIT)
#pragma pack
#elif defined(CICS_HPUX)
#pragma HP_ALIGN HPUX_NATURAL
#else
#pragma pack()
#endif

#ifdef __cplusplus
  }
#endif

#endif /* _CICSEPIX_H */
