/* Module Name   : ecix1.c

   Product       : CICS Transaction Gateway

   Description   : ECI Exit Library Sample

                   This is a sample ECI exits library.  It performs no useful
                   function but does provide a skeleton containing all the
                   exit functions with details of valid return codes and their
                   meanings.

                   When writing exits, the following rules must be obeyed:
                   1. No EPI or ECI calls must be made from the exit
                   2. Exits should contain the minimum possible executable
                      code (to minimize impact on performance)
                   3. All exit code must be re-entrant and thread safe.
                   4. The primary entry point to this module must be named
                      CICS_ECIEXITINIT.  The names of the actual exits may
                      be changed but the parameter lists must not be changed.
                   5. An ECI user exit program should contain valid entry
                      points for all of the user exit functions, with the
                      exception of CICS_EciSetProgramAliasExit which is
                      optional.

 Licensed Materials - Property of IBM  
  
 5724-I81,5725-B65 
  
 (C) Copyright IBM Corp. 1999, 2014 All Rights Reserved.  
  
 US Government Users Restricted Rights - Use, duplication or  
 disclosure restricted by GSA ADP Schedule Contract with  
 IBM Corp.  
    
    Status: Version 9 Release 1 

                   The following code is sample code created by IBM
                   Corporation.  This sample code is not part of any standard
                   IBM product and is provided to you solely for the purpose
                   of assisting you in the development of your applications.
                   The code is provided 'AS IS', without warranty or
                   condition of any kind.  IBM shall not be liable for any
                   damages arising out of your use of the sample code, even
                   if IBM has been advised of the possibility of such damages. */


/* HEADER FILES */

#include <cicstype.h>
#include <cics_eci.h>
#include <cicsecix.h>

#define copyright_notice copyright4cecix1
char version4cecix1[] =
"\nCICS Transaction Gateway"
" Version 9.1.0.4"
" Build Level c910-20231012-1355" ;

char copyright_notice[] =
"\n"
"Licensed Materials - Property of IBM. "
"\n"
"5724-I81,5725-B65 "
"\n"
"(c) Copyright IBM Corp. 1999, 2014 All Rights Reserved. "
"\n"
"US Government Users Restricted Rights - Use, duplication or "
"disclosure restricted by GSA ADP Schedule Contract with "
"IBM Corp."
"\n";

/* Main entry point of dynamically loaded object.
   This function returns a structure of addresses of exit functions.
   Note that ALL exits must be defined if any are defined.
   If any exit address is set to NULL no exits will be called. */

#if defined(CICS_W32)
  #define CICSEXPORT __declspec ( dllexport )
#else
  #define CICSEXPORT
#endif

void CICSEXPORT CICSEXIT CICS_ECIEXITINIT(CICS_EciExitList_t CICSPTR *ExitList)
{
    ExitList->InitializeExit    = (CICS_EciInitializeExitFn CICSPTR *)&CICS_EciInitializeExit;
    ExitList->TerminateExit     = (CICS_EciTerminateExitFn CICSPTR *)&CICS_EciTerminateExit;
    ExitList->ExternalCallExit1 = (CICS_EciExternalCallExit1Fn CICSPTR *)&CICS_EciExternalCallExit1;
    ExitList->ExternalCallExit2 = (CICS_EciExternalCallExit2Fn CICSPTR *)&CICS_EciExternalCallExit2;
    ExitList->SystemIdExit      = (CICS_EciSystemIdExitFn CICSPTR *)&CICS_EciSystemIdExit;
    ExitList->DataSendExit      = (CICS_EciDataSendExitFn CICSPTR *)&CICS_EciDataSendExit;
    ExitList->DataReturnExit    = (CICS_EciDataReturnExitFn CICSPTR *)&CICS_EciDataReturnExit;
}


/* InitializeExit:
   Called immediately after successful initialization.

   Permitted return values:
     CICS_EXIT_OK               No problems - call exits where appropriate
     CICS_EXIT_NO_EXITS         No problems - do not call subsequent exits
                                    (used by shipped dummy library)
     CICS_EXIT_CANT_INIT_EXITS  Exits cannot be used due to an
                                    (unspecified) error condition
     CICS_EXIT_USER_BASE + N    User specified value

   Notes:
     Any value except CICS_EXIT_OK will inhibit the calling of subsequent exits */

cics_sshort_t CICSEXIT CICS_EciInitializeExit( cics_sshort_t             Version,
                                               CICS_EciAnchor_t CICSPTR *Anchor)

{
        /* Anchor is provided to allow user exit code to share storage.
           In this example, the shared storage is an integer containing -1. */

        *Anchor = (void *)(-1);
        return CICS_EXIT_NO_EXITS;
}


/* TerminateExit:
   Called immediately prior to the destruction of the
   ECI data structures in the atexit processing.

   Permitted return values:
     CICS_EXIT_OK               No problems
     CICS_EXIT_BAD_ANCHOR       Invalid anchor value supplied
     CICS_EXIT_BAD_STORAGE      Exit cannot tidy up its storage
     CICS_EXIT_USER_BASE + N    User specified value */

cics_sshort_t CICSEXIT CICS_EciTerminateExit( CICS_EciAnchor_t Anchor )
{
        return CICS_EXIT_OK;
}


/* SystemIdExit:
   Called if the ECI would return ECI_ERR_NO_CICS, ECI_ERR_UNKNOWN_SERVER
   or ECI_ERR_SECURITY_ERROR.  This allows the choice of an alternative
   system id.  This is the only exit allowed to request a retry.
   This exit will be called repeatedly until it returns a system which can
   successfully fulfill the request or until it returns CICS_EXIT_OK.
   CICS_EXIT_GIVE_UP can be used to force a non-error return from this loop.

    Permitted return values:
     CICS_EXIT_OK               No problems
     CICS_EXIT_BAD_ANCHOR       Invalid anchor value supplied
     CICS_EXIT_BAD_PARM         One or more parameters bad
     CICS_EXIT_GIVE_UP          There are no more alternative systems
     CICS_EXIT_USER_BASE + N    User specified value */

cics_sshort_t CICSEXIT CICS_EciSystemIdExit( CICS_EciAnchor_t   Anchor,
                                             CICS_EciToken_t    Token,
                                             ECI_PARMS CICSPTR *ParmPtr,
                                             cics_sshort_t      Reason)
{

       /*
        * If the workload manager is active ( Windows only ), and user exits
        * are being stacked with workload manger exits, this exit will only
        * be called if the workload manager returns CICS_EXIT_OK.
        */
        return CICS_EXIT_GIVE_UP;
}


/* ExternalCallExit1:
   Called after parameter verification for all LINK and STATUS calls.
   This exit allows system selection.

    Permitted return values:
     CICS_EXIT_OK               No problems
     CICS_EXIT_BAD_ANCHOR       Invalid anchor value supplied
     CICS_EXIT_BAD_PARM         One or more parameters bad
     CICS_EXIT_USER_BASE + N    User specified value */

cics_sshort_t CICSEXIT CICS_EciExternalCallExit1( CICS_EciAnchor_t   Anchor,
                                                  CICS_EciToken_t    Token,
                                                  ECI_PARMS CICSPTR *ParmPtr)
{
        return CICS_EXIT_OK;
}


/* ExternalCallExit2:
   Called prior to returning from all LINK and STATUS calls.

    Permitted return values:
     CICS_EXIT_OK               No problems
     CICS_EXIT_BAD_ANCHOR       Invalid anchor value supplied
     CICS_EXIT_BAD_PARM         One or more parameters bad
     CICS_EXIT_USER_BASE + N    User specified value */

cics_sshort_t CICSEXIT CICS_EciExternalCallExit2( CICS_EciAnchor_t   Anchor,
                                                  CICS_EciToken_t    Token,
                                                  ECI_PARMS CICSPTR *ParmPtr)
{
        return CICS_EXIT_OK;
}


/* DataSendExit:
   Called immediately before data is sent to the host.

    Permitted return values:
     CICS_EXIT_OK               No problems
     CICS_EXIT_BAD_ANCHOR       Invalid anchor value supplied
     CICS_EXIT_BAD_PARM         One or more parameters bad
     CICS_EXIT_USER_BASE + N    User specified value */

cics_sshort_t CICSEXIT CICS_EciDataSendExit( CICS_EciAnchor_t Anchor,
                                             CICS_EciToken_t  Token)
{
        return CICS_EXIT_OK;
}


/* DataReturnExit:
   Called immediately after data is received from the host.

    Permitted return values:
     CICS_EXIT_OK               No problems
     CICS_EXIT_BAD_ANCHOR       Invalid anchor value supplied
     CICS_EXIT_BAD_PARM         One or more parameters bad
     CICS_EXIT_USER_BASE + N    User specified value */

cics_sshort_t CICSEXIT CICS_EciDataReturnExit( CICS_EciAnchor_t   Anchor,
                                               CICS_EciToken_t    Token,
                                               ECI_PARMS CICSPTR *ParmPtr)
{
        return CICS_EXIT_OK;
}
